@@ -28,22 +28,23 @@
 #include <sstream>
 
 static const float SQUISH_TIME = 2;
-  
+
 static const float X_OFFSCREEN_DISTANCE = 1280;
 static const float Y_OFFSCREEN_DISTANCE = 800;
 
-BadGuy::BadGuy(const Vector& pos, const std::string& sprite_name, int layer) :
-  MovingSprite(pos, sprite_name, layer, COLGROUP_DISABLED), 
+BadGuy::BadGuy(const Vector& pos, const std::string& sprite_name_, int layer_) :
+  MovingSprite(pos, sprite_name_, layer_, COLGROUP_DISABLED),
   physic(),
-  countMe(true), 
+  countMe(true),
   is_initialized(false),
   start_position(),
-  dir(LEFT), 
-  start_dir(AUTO), 
-  frozen(false), 
+  dir(LEFT),
+  start_dir(AUTO),
+  frozen(false),
   ignited(false),
+  in_water(false),
   dead_script(),
-  state(STATE_INIT), 
+  state(STATE_INIT),
   is_active_flag(),
   state_timer(),
   on_ground_flag(false),
@@ -52,53 +53,57 @@ BadGuy::BadGuy(const Vector& pos, const std::string& sprite_name, int layer) :
 {
   start_position = bbox.p1;
 
-  sound_manager->preload("sounds/squish.wav");
-  sound_manager->preload("sounds/fall.wav");
+  SoundManager::current()->preload("sounds/squish.wav");
+  SoundManager::current()->preload("sounds/fall.wav");
+  SoundManager::current()->preload("sounds/splash.wav");
 
   dir = (start_dir == AUTO) ? LEFT : start_dir;
 }
 
-BadGuy::BadGuy(const Vector& pos, Direction direction, const std::string& sprite_name, int layer) :
-  MovingSprite(pos, sprite_name, layer, COLGROUP_DISABLED), 
+BadGuy::BadGuy(const Vector& pos, Direction direction, const std::string& sprite_name_, int layer_) :
+  MovingSprite(pos, sprite_name_, layer_, COLGROUP_DISABLED),
   physic(),
-  countMe(true), 
-  is_initialized(false), 
+  countMe(true),
+  is_initialized(false),
   start_position(),
-  dir(direction), 
-  start_dir(direction), 
-  frozen(false), 
+  dir(direction),
+  start_dir(direction),
+  frozen(false),
   ignited(false),
+  in_water(false),
   dead_script(),
-  state(STATE_INIT), 
+  state(STATE_INIT),
   is_active_flag(),
   state_timer(),
-  on_ground_flag(false), 
+  on_ground_flag(false),
   floor_normal(),
   colgroup_active(COLGROUP_MOVING)
 {
   start_position = bbox.p1;
 
-  sound_manager->preload("sounds/squish.wav");
-  sound_manager->preload("sounds/fall.wav");
+  SoundManager::current()->preload("sounds/squish.wav");
+  SoundManager::current()->preload("sounds/fall.wav");
+  SoundManager::current()->preload("sounds/splash.wav");
 
   dir = (start_dir == AUTO) ? LEFT : start_dir;
 }
 
-BadGuy::BadGuy(const Reader& reader, const std::string& sprite_name, int layer) :
-  MovingSprite(reader, sprite_name, layer, COLGROUP_DISABLED), 
+BadGuy::BadGuy(const Reader& reader, const std::string& sprite_name_, int layer_) :
+  MovingSprite(reader, sprite_name_, layer_, COLGROUP_DISABLED),
   physic(),
-  countMe(true), 
-  is_initialized(false), 
+  countMe(true),
+  is_initialized(false),
   start_position(),
-  dir(LEFT), 
+  dir(LEFT),
   start_dir(AUTO),
-  frozen(false), 
-  ignited(false), 
+  frozen(false),
+  ignited(false),
+  in_water(false),
   dead_script(),
-  state(STATE_INIT), 
+  state(STATE_INIT),
   is_active_flag(),
   state_timer(),
-  on_ground_flag(false), 
+  on_ground_flag(false),
   floor_normal(),
   colgroup_active(COLGROUP_MOVING)
 {
@@ -111,8 +116,9 @@ BadGuy::BadGuy(const Reader& reader, const std::string& sprite_name, int layer)
 
   reader.get("dead-script", dead_script);
 
-  sound_manager->preload("sounds/squish.wav");
-  sound_manager->preload("sounds/fall.wav");
+  SoundManager::current()->preload("sounds/squish.wav");
+  SoundManager::current()->preload("sounds/fall.wav");
+  SoundManager::current()->preload("sounds/splash.wav");
 
   dir = (start_dir == AUTO) ? LEFT : start_dir;
 }
@@ -126,7 +132,7 @@ BadGuy::draw(DrawingContext& context)
     return;
   if(state == STATE_FALLING) {
     DrawingEffect old_effect = context.get_drawing_effect();
-    context.set_drawing_effect((DrawingEffect) (old_effect | VERTICAL_FLIP));
+    context.set_drawing_effect(old_effect | VERTICAL_FLIP);
     sprite->draw(context, get_pos(), layer);
     context.set_drawing_effect(old_effect);
   } else {
@@ -140,6 +146,15 @@ BadGuy::update(float elapsed_time)
   if(!Sector::current()->inside(bbox)) {
     is_active_flag = false;
     remove_me();
+    if(countMe) {
+      // get badguy name from sprite_name ignoring path and extension
+      std::string badguy = sprite_name.substr(0, sprite_name.length() - 7);
+      int path_chars = badguy.rfind("/",badguy.length());
+      badguy = badguy.substr(path_chars + 1, badguy.length() - path_chars);
+      // log warning since badguys_killed can no longer reach total_badguys
+      std::string current_level = "[" + Sector::current()->get_level()->filename + "] ";
+      log_warning << current_level << "Counted badguy " << badguy << " starting at " << start_position << " has left the sector" <<std::endl;;
+    }
     return;
   }
   if ((state != STATE_INACTIVE) && is_offscreen()) {
@@ -209,6 +224,8 @@ void
 BadGuy::active_update(float elapsed_time)
 {
   movement = physic.get_movement(elapsed_time);
+  if(frozen)
+    sprite->stop_animation();
 }
 
 void
@@ -219,6 +236,19 @@ BadGuy::inactive_update(float )
 void
 BadGuy::collision_tile(uint32_t tile_attributes)
 {
+  // Don't kill badguys that have already been killed
+  if (!is_active()) return;
+
+  if(tile_attributes & Tile::WATER && !is_in_water())
+  {
+    in_water = true;
+    SoundManager::current()->play("sounds/splash.wav", get_pos());
+  }
+  if(!(tile_attributes & Tile::WATER) && is_in_water())
+  {
+    in_water = false;
+  }
+
   if(tile_attributes & Tile::HURTS) {
     if (tile_attributes & Tile::FIRE) {
       if (is_flammable()) ignite();
@@ -258,11 +288,20 @@ BadGuy::collision(GameObject& other, const CollisionHit& hit)
 
     // hit from above?
     if (player->get_bbox().p2.y < (bbox.p1.y + 16)) {
+      if(player->is_stone()) {
+        kill_fall();
+        return FORCE_MOVE;
+      }
       if(collision_squished(*player)) {
         return FORCE_MOVE;
       }
     }
 
+    if(player->is_stone()) {
+      collision_solid(hit);
+      return FORCE_MOVE;
+    }
+
     return collision_player(*player, hit);
   }
 
@@ -289,8 +328,11 @@ BadGuy::collision_player(Player& player, const CollisionHit& )
     return ABORT_MOVE;
   }
 
+  //TODO: unfreeze timer
   if(frozen)
-    unfreeze();
+    //unfreeze();
+    return FORCE_MOVE;
+
   player.kill(false);
   return FORCE_MOVE;
 }
@@ -302,9 +344,19 @@ BadGuy::collision_badguy(BadGuy& , const CollisionHit& )
 }
 
 bool
-BadGuy::collision_squished(GameObject& )
+BadGuy::collision_squished(GameObject& object)
 {
-  return false;
+  // frozen badguys can be killed with butt-jump
+  if(frozen)
+  {
+    Player* player = dynamic_cast<Player*>(&object);
+    if(player && (player->does_buttjump)) {
+      player->bounce(*this);
+      kill_fall();//TODO: shatter frozen badguys
+      return true;
+    }
+  }
+    return false;
 }
 
 HitResponse
@@ -356,7 +408,9 @@ BadGuy::collision_bullet(Bullet& bullet, const CollisionHit& hit)
 void
 BadGuy::kill_squished(GameObject& object)
 {
-  sound_manager->play("sounds/squish.wav", get_pos());
+  if (!is_active()) return;
+
+  SoundManager::current()->play("sounds/squish.wav", get_pos());
   physic.enable_gravity(true);
   physic.set_velocity_x(0);
   physic.set_velocity_y(0);
@@ -374,12 +428,18 @@ BadGuy::kill_squished(GameObject& object)
 void
 BadGuy::kill_fall()
 {
-  sound_manager->play("sounds/fall.wav", get_pos());
+  if (!is_active()) return;
+
+  SoundManager::current()->play("sounds/fall.wav", get_pos());
   physic.set_velocity_y(0);
   physic.set_acceleration_y(0);
   physic.enable_gravity(true);
   set_state(STATE_FALLING);
 
+  // Set the badguy layer to be the foremost, so that
+  // this does not reveal secret tilemaps:
+  layer = Sector::current()->get_foremost_layer() + 1;
+
   // start dead-script
   run_dead_script();
 }
@@ -391,7 +451,7 @@ BadGuy::run_dead_script()
     Sector::current()->get_level()->stats.badguys++;
 
   countMe = false;
-   
+
   // start dead-script
   if(dead_script != "") {
     std::istringstream stream(dead_script);
@@ -400,14 +460,14 @@ BadGuy::run_dead_script()
 }
 
 void
-BadGuy::set_state(State state)
+BadGuy::set_state(State state_)
 {
-  if(this->state == state)
+  if(this->state == state_)
     return;
 
   State laststate = this->state;
-  this->state = state;
-  switch(state) {
+  this->state = state_;
+  switch(state_) {
     case STATE_SQUISHED:
       state_timer.start(SQUISH_TIME);
       break;
@@ -457,8 +517,8 @@ BadGuy::try_activate()
 
       // if starting direction was set to AUTO, this is our chance to re-orient the badguy
       if (start_dir == AUTO) {
-        Player* player = get_nearest_player();
-        if (player && (player->get_bbox().p1.x > get_bbox().p2.x)) {
+        Player* player_ = get_nearest_player();
+        if (player_ && (player_->get_bbox().p1.x > get_bbox().p2.x)) {
           dir = RIGHT;
         } else {
           dir = LEFT;
@@ -530,6 +590,15 @@ BadGuy::freeze()
 {
   set_group(COLGROUP_MOVING_STATIC);
   frozen = true;
+
+  if(sprite->has_action("iced-left"))
+    sprite->set_action(dir == LEFT ? "iced-left" : "iced-right", 1);
+  // when no iced action exists, default to shading badguy blue
+  else
+  {
+    sprite->set_color(Color(0.60, 0.72, 0.88f));
+    sprite->stop_animation();
+  }
 }
 
 void
@@ -537,6 +606,13 @@ BadGuy::unfreeze()
 {
   set_group(colgroup_active);
   frozen = false;
+
+  // restore original color if needed
+  if(!sprite->has_action("iced-left"))
+  {
+    sprite->set_color(Color(1.00, 1.00, 1.00f));
+    sprite->set_animation_loops();
+  }
 }
 
 bool
@@ -551,6 +627,12 @@ BadGuy::is_frozen() const
   return frozen;
 }
 
+bool
+BadGuy::is_in_water() const
+{
+  return in_water;
+}
+
 void
 BadGuy::ignite()
 {
@@ -573,12 +655,12 @@ BadGuy::is_ignited() const
 {
   return ignited;
 }
-  
-void 
-BadGuy::set_colgroup_active(CollisionGroup group)
+
+void
+BadGuy::set_colgroup_active(CollisionGroup group_)
 {
-  this->colgroup_active = group;
-  if (state == STATE_ACTIVE) set_group(group); 
+  this->colgroup_active = group_;
+  if (state == STATE_ACTIVE) set_group(group_);
 }
 
 /* EOF */
