@@ -24,7 +24,7 @@ static const float FISH_JUMP_POWER = -600;
 static const float FISH_WAIT_TIME = 1;
 
 Fish::Fish(const Reader& reader) :
-  BadGuy(reader, "images/creatures/fish/fish.sprite", LAYER_TILES-1), 
+  BadGuy(reader, "images/creatures/fish/fish.sprite", LAYER_TILES-1),
   waiting(),
   stop_y(0)
 {
@@ -32,7 +32,7 @@ Fish::Fish(const Reader& reader) :
 }
 
 Fish::Fish(const Vector& pos) :
-  BadGuy(pos, "images/creatures/fish/fish.sprite", LAYER_TILES-1), 
+  BadGuy(pos, "images/creatures/fish/fish.sprite", LAYER_TILES-1),
   waiting(),
   stop_y(0)
 {
@@ -67,9 +67,9 @@ Fish::draw(DrawingContext& context)
 }
 
 HitResponse
-Fish::hit(const CollisionHit& hit)
+Fish::hit(const CollisionHit& hit_)
 {
-  if(hit.top) {
+  if(hit_.top) {
     physic.set_velocity_y(0);
   }
 
@@ -138,6 +138,7 @@ Fish::freeze()
 {
   BadGuy::freeze();
   sprite->set_action(physic.get_velocity_y() < 0 ? "iced" : "iced-down");
+  sprite->set_color(Color(1.0f, 1.0f, 1.0f));
   waiting.stop();
 }
 
