@@ -33,6 +33,8 @@ public:
   void kill_fall();
   void explode();
 
+  virtual void draw(DrawingContext& context);
+
 private:
   void try_activate();
   HitResponse hit(const CollisionHit& hit);
@@ -44,7 +46,8 @@ private:
   Timer movement_timer;
   Timer lifetime;
   int direction;
-  State state;
+  Color light;
+  SpritePtr lightsprite;
 };
 
 #endif
