@@ -19,6 +19,7 @@
 #include "audio/sound_manager.hpp"
 #include "object/player.hpp"
 #include "sprite/sprite.hpp"
+#include "sprite/sprite_manager.hpp"
 #include "supertux/object_factory.hpp"
 
 #include <math.h>
@@ -30,31 +31,31 @@ const float NOKICK_TIME = 0.1f;
 }
 
 MrIceBlock::MrIceBlock(const Reader& reader) :
-  WalkingBadguy(reader, "images/creatures/mr_iceblock/mr_iceblock.sprite", "left", "right"), 
-  ice_state(ICESTATE_NORMAL), 
+  WalkingBadguy(reader, "images/creatures/mr_iceblock/mr_iceblock.sprite", "left", "right"),
+  ice_state(ICESTATE_NORMAL),
   nokick_timer(),
   flat_timer(),
   squishcount(0)
 {
   walk_speed = 80;
   max_drop_height = 600;
-  sound_manager->preload("sounds/iceblock_bump.wav");
-  sound_manager->preload("sounds/stomp.wav");
-  sound_manager->preload("sounds/kick.wav");
+  SoundManager::current()->preload("sounds/iceblock_bump.wav");
+  SoundManager::current()->preload("sounds/stomp.wav");
+  SoundManager::current()->preload("sounds/kick.wav");
 }
 
 MrIceBlock::MrIceBlock(const Vector& pos, Direction d) :
-  WalkingBadguy(pos, d, "images/creatures/mr_iceblock/mr_iceblock.sprite", "left", "right"), 
-  ice_state(ICESTATE_NORMAL), 
+  WalkingBadguy(pos, d, "images/creatures/mr_iceblock/mr_iceblock.sprite", "left", "right"),
+  ice_state(ICESTATE_NORMAL),
   nokick_timer(),
   flat_timer(),
   squishcount(0)
 {
   walk_speed = 80;
   max_drop_height = 600;
-  sound_manager->preload("sounds/iceblock_bump.wav");
-  sound_manager->preload("sounds/stomp.wav");
-  sound_manager->preload("sounds/kick.wav");
+  SoundManager::current()->preload("sounds/iceblock_bump.wav");
+  SoundManager::current()->preload("sounds/stomp.wav");
+  SoundManager::current()->preload("sounds/kick.wav");
 }
 
 void
@@ -105,7 +106,7 @@ MrIceBlock::collision_solid(const CollisionHit& hit)
     case ICESTATE_KICKED: {
       if((hit.right && dir == RIGHT) || (hit.left && dir == LEFT)) {
         dir = (dir == LEFT) ? RIGHT : LEFT;
-        sound_manager->play("sounds/iceblock_bump.wav", get_pos());
+        SoundManager::current()->play("sounds/iceblock_bump.wav", get_pos());
         physic.set_velocity_x(-physic.get_velocity_x()*.975);
       }
       this->set_action(dir == LEFT ? "flat-left" : "flat-right", /* loops = */ -1);
@@ -223,12 +224,12 @@ MrIceBlock::collision_squished(GameObject& object)
 }
 
 void
-MrIceBlock::set_state(IceState state, bool up)
+MrIceBlock::set_state(IceState state_, bool up)
 {
-  if(ice_state == state)
+  if(ice_state == state_)
     return;
 
-  switch(state) {
+  switch(state_) {
     case ICESTATE_NORMAL:
       this->set_action(dir == LEFT ? "left" : "right", /* loops = */ -1);
       WalkingBadguy::initialize();
@@ -237,7 +238,7 @@ MrIceBlock::set_state(IceState state, bool up)
       if(up) {
         physic.set_velocity_y(-KICKSPEED);
       } else {
-        sound_manager->play("sounds/stomp.wav", get_pos());
+        SoundManager::current()->play("sounds/stomp.wav", get_pos());
         physic.set_velocity_x(0);
         physic.set_velocity_y(0);
       }
@@ -245,7 +246,7 @@ MrIceBlock::set_state(IceState state, bool up)
       flat_timer.start(4);
       break;
     case ICESTATE_KICKED:
-      sound_manager->play("sounds/kick.wav", get_pos());
+      SoundManager::current()->play("sounds/kick.wav", get_pos());
 
       physic.set_velocity_x(dir == LEFT ? -KICKSPEED : KICKSPEED);
       this->set_action(dir == LEFT ? "flat-left" : "flat-right", /* loops = */ -1);
@@ -258,26 +259,26 @@ MrIceBlock::set_state(IceState state, bool up)
     default:
       assert(false);
   }
-  ice_state = state;
+  ice_state = state_;
 }
 
 void
-MrIceBlock::grab(MovingObject&, const Vector& pos, Direction dir)
+MrIceBlock::grab(MovingObject&, const Vector& pos, Direction dir_)
 {
   movement = pos - get_pos();
-  this->dir = dir;
-  this->set_action(dir == LEFT ? "flat-left" : "flat-right", /* loops = */ -1);
+  this->dir = dir_;
+  this->set_action(dir_ == LEFT ? "flat-left" : "flat-right", /* loops = */ -1);
   set_state(ICESTATE_GRABBED);
   set_colgroup_active(COLGROUP_DISABLED);
 }
 
 void
-MrIceBlock::ungrab(MovingObject& , Direction dir)
+MrIceBlock::ungrab(MovingObject& , Direction dir_)
 {
-  if(dir == UP) {
+  if(dir_ == UP) {
     set_state(ICESTATE_FLAT, true);
   } else {
-    this->dir = dir;
+    this->dir = dir_;
     set_state(ICESTATE_KICKED);
   }
   set_colgroup_active(COLGROUP_MOVING);
@@ -289,4 +290,11 @@ MrIceBlock::is_portable() const
   return ice_state == ICESTATE_FLAT;
 }
 
+SmartBlock::SmartBlock(const Reader& reader) :
+  MrIceBlock(reader)
+{
+  max_drop_height = 16;
+  sprite = SpriteManager::current()->create("images/creatures/mr_iceblock/smart_block/smart_block.sprite");
+}
+
 /* EOF */
