@@ -20,7 +20,7 @@
 #include "badguy/walking_badguy.hpp"
 #include "object/portable.hpp"
 
-class MrIceBlock : public WalkingBadguy, 
+class MrIceBlock : public WalkingBadguy,
                    public Portable
 {
 public:
@@ -60,6 +60,12 @@ private:
   int squishcount;
 };
 
+class SmartBlock : public MrIceBlock
+{
+public:
+  SmartBlock(const Reader& reader);
+};
+
 #endif
 
 /* EOF */
