@@ -31,6 +31,10 @@ public:
   void collision_solid(const CollisionHit& hit);
   void kill_fall();
 
+  void freeze();
+  void unfreeze();
+  bool is_freezable() const;
+
 protected:
   bool is_above_player (void);
   void active_update (float elapsed_time);
