@@ -20,12 +20,10 @@
 #include "sprite/sprite.hpp"
 #include "supertux/object_factory.hpp"
 
-static const float WALKSPEED = 80;
-
 SSpiky::SSpiky(const Reader& reader)
   : WalkingBadguy(reader, "images/creatures/spiky/sleepingspiky.sprite", "left", "right"), state(SSPIKY_SLEEPING)
 {
-  walk_speed = WALKSPEED;
+  walk_speed = 80;
   max_drop_height = 600;
 }
 
@@ -101,7 +99,6 @@ void
 SSpiky::freeze()
 {
   WalkingBadguy::freeze();
-  sprite->set_action(dir == LEFT ? "iced-left" : "iced-right");
   state = SSPIKY_WALKING; // if we get hit while sleeping, wake up :)
 }
 
