@@ -1,5 +1,5 @@
 //  SuperTux
-//  Copyright (C) 2009 Ingo Ruhnke <grumbel@gmx.de>
+//  Copyright (C) 2009 Ingo Ruhnke <grumbel@gmail.com>
 //
 //  This program is free software: you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -22,9 +22,9 @@
 class Brick : public Block
 {
 public:
-  Brick(const Vector& pos, int data);
+  Brick(const Vector& pos, int data, const std::string& spriteName);
 
-  void try_break(Player* player = false);
+  void try_break(Player* player);
   HitResponse collision(GameObject& other, const CollisionHit& hit);
 
 protected:
