@@ -20,8 +20,11 @@
 #include "badguy/badguy.hpp"
 #include "badguy/walking_badguy.hpp"
 #include "math/random_generator.hpp"
+#include "object/particles.hpp"
 #include "object/player.hpp"
 #include "object/sprite_particle.hpp"
+#include "sprite/sprite.hpp"
+#include "sprite/sprite_manager.hpp"
 #include "supertux/object_factory.hpp"
 #include "supertux/sector.hpp"
 
@@ -31,19 +34,29 @@ Explosion::Explosion(const Vector& pos) :
   MovingSprite(pos, "images/objects/explosion/explosion.sprite", LAYER_OBJECTS+40, COLGROUP_MOVING),
   hurt(true),
   push(false),
-  state(STATE_WAITING)
+  state(STATE_WAITING),
+  light(0.0f,0.0f,0.0f),
+  lightsprite(SpriteManager::current()->create("images/objects/lightmap_light/lightmap_light-large.sprite"))
 {
-  sound_manager->preload("sounds/explosion.wav");
+  SoundManager::current()->preload("sounds/explosion.wav");
+  SoundManager::current()->preload("sounds/firecracker.ogg");
   set_pos(get_pos() - (get_bbox().get_middle() - get_pos()));
+  lightsprite->set_blend(Blend(GL_SRC_ALPHA, GL_ONE));
+  lightsprite->set_color(Color(0.6f, 0.6f, 0.6f));
 }
 
 Explosion::Explosion(const Reader& reader) :
   MovingSprite(reader, "images/objects/explosion/explosion.sprite", LAYER_OBJECTS+40, COLGROUP_MOVING),
   hurt(true),
   push(false),
-  state(STATE_WAITING)
+  state(STATE_WAITING),
+  light(0.0f,0.0f,0.0f),
+  lightsprite(SpriteManager::current()->create("images/objects/lightmap_light/lightmap_light-large.sprite"))
 {
-  sound_manager->preload("sounds/explosion.wav");
+  SoundManager::current()->preload("sounds/explosion.wav");
+  SoundManager::current()->preload("sounds/firecracker.ogg");
+  lightsprite->set_blend(Blend(GL_SRC_ALPHA, GL_ONE));
+  lightsprite->set_color(Color(0.6f, 0.6f, 0.6f));
 }
 
 void
@@ -53,24 +66,15 @@ Explosion::explode()
     return;
   state = STATE_EXPLODING;
 
-  set_action("default", 1);
+  set_action(hurt ? "default" : "pop", 1);
   sprite->set_animation_loops(1); //TODO: this is necessary because set_action will not set "loops" when "action" is the default action
-  sound_manager->play("sounds/explosion.wav", get_pos());
+  sprite->set_angle(graphicsRandom.randf(0, 360)); // a random rotation on the sprite to make explosions appear more random
+  SoundManager::current()->play(hurt ? "sounds/explosion.wav" : "sounds/firecracker.ogg", get_pos());
 
-#if 0
   // spawn some particles
-  // TODO: provide convenience function in MovingSprite or MovingObject?
-  for (int i = 0; i < 100; i++) {
-    Vector ppos = bbox.get_middle();
-    float angle = graphicsRandom.randf(-M_PI_2, M_PI_2);
-    float velocity = graphicsRandom.randf(450, 900);
-    float vx = sin(angle)*velocity;
-    float vy = -cos(angle)*velocity;
-    Vector pspeed = Vector(vx, vy);
-    Vector paccel = Vector(0, 1000);
-    Sector::current()->add_object(new SpriteParticle("images/objects/particles/explosion.sprite", "default", ppos, ANCHOR_MIDDLE, pspeed, paccel, LAYER_OBJECTS-1));
-  }
-#endif
+  int pnumber = push ? 8 : 100;
+  Sector::current()->add_object(std::make_shared<Particles>(
+    bbox.get_middle(), -360, 360, 450, 900, Vector(0, 1000), pnumber, Color(.4f, .4f, .4f), 3, .8f, LAYER_OBJECTS-1));
 
   if (push) {
     Vector center = get_bbox ().get_middle ();
@@ -108,7 +112,7 @@ Explosion::explode()
   } /* if (push) */
 }
 
-void 
+void
 Explosion::update(float )
 {
   switch(state) {
@@ -123,6 +127,21 @@ Explosion::update(float )
   }
 }
 
+void
+Explosion::draw(DrawingContext& context)
+{
+  //Draw the Sprite.
+  sprite->draw(context, get_pos(), LAYER_OBJECTS+40);
+  //Explosions produce light (if ambient light is not maxed)
+  context.get_light( get_bbox().get_middle(), &light);
+  if (light.red + light.green + light.blue < 3.0){
+    context.push_target();
+    context.set_target(DrawingContext::LIGHTMAP);
+    lightsprite->draw(context, get_bbox().get_middle(), 0);
+    context.pop_target();
+  }
+}
+
 HitResponse
 Explosion::collision(GameObject& other, const CollisionHit& )
 {
