@@ -27,12 +27,16 @@ public:
   GrowUp(Direction direction = RIGHT);
 
   virtual void update(float elapsed_time);
+  virtual void draw(DrawingContext& context);
   virtual void collision_solid(const CollisionHit& hit);
   virtual HitResponse collision(GameObject& other, const CollisionHit& hit);
   void do_jump();
 
 private:
   Physic physic;
+  Color light;
+  SpritePtr shadesprite;
+  SpritePtr lightsprite;
 };
 
 #endif
