@@ -19,7 +19,7 @@
 
 #include "audio/sound_manager.hpp"
 #include "badguy/badguy.hpp"
-#include "control/joystickkeyboardcontroller.hpp"
+#include "control/input_manager.hpp"
 #include "math/random_generator.hpp"
 #include "object/bullet.hpp"
 #include "object/camera.hpp"
@@ -30,7 +30,7 @@
 #include "object/sprite_particle.hpp"
 #include "scripting/squirrel_util.hpp"
 #include "supertux/game_session.hpp"
-#include "supertux/globals.hpp"
+#include "supertux/gameconfig.hpp"
 #include "supertux/sector.hpp"
 #include "supertux/tile.hpp"
 #include "trigger/climbable.hpp"
@@ -40,9 +40,10 @@
 //#define SWIMMING
 
 namespace {
-static const int TILES_FOR_BUTTJUMP = 3;
 static const float BUTTJUMP_MIN_VELOCITY_Y = 400.0f;
 static const float SHOOTING_TIME = .150f;
+static const float GLIDE_TIME_PER_FLOWER = 0.5f;
+static const float STONE_TIME_PER_FLOWER = 2.0f;
 
 /** number of idle stages, including standing */
 static const unsigned int IDLE_STAGE_COUNT = 5;
@@ -63,7 +64,7 @@ static const std::string IDLE_STAGES[] =
 /** acceleration in horizontal direction when walking
  * (all accelerations are in  pixel/s^2) */
 static const float WALK_ACCELERATION_X = 300;
-/** acceleration in horizontal direction when running */ 
+/** acceleration in horizontal direction when running */
 static const float RUN_ACCELERATION_X = 400;
 /** acceleration when skidding */
 static const float SKID_XM = 200;
@@ -73,10 +74,14 @@ static const float SKID_TIME = .3f;
 static const float MAX_WALK_XM = 230;
 /** maximum run velocity (pixel/s) */
 static const float MAX_RUN_XM = 320;
+/** bonus run velocity addition (pixel/s) */
+static const float BONUS_RUN_XM = 80;
 /** maximum horizontal climb velocity */
 static const float MAX_CLIMB_XM = 96;
 /** maximum vertical climb velocity */
 static const float MAX_CLIMB_YM = 128;
+/** maximum vertical glide velocity */
+static const float MAX_GLIDE_YM = 128;
 /** instant velocity when tux starts to walk */
 static const float WALK_SPEED = 100;
 
@@ -88,8 +93,6 @@ static const float ICE_ACCELERATION_MULTIPLIER = 0.25f;
 
 /** time of the kick (kicking mriceblock) animation */
 static const float KICK_TIME = .3f;
-/** time of tux cheering (currently unused) */
-static const float CHEER_TIME = 1.0f;
 
 /** if Tux cannot unduck for this long, he will get hurt */
 static const float UNDUCK_HURT_TIME = 0.25f;
@@ -109,24 +112,29 @@ static const float DUCKED_TUX_HEIGHT = 31.8f;
 bool no_water = true;
 }
 
-Player::Player(PlayerStatus* _player_status, const std::string& name) :
+Player::Player(PlayerStatus* _player_status, const std::string& name_) :
   deactivated(),
   controller(),
-  scripting_controller(0), 
-  player_status(_player_status), 
+  scripting_controller(),
+  player_status(_player_status),
   duck(),
   dead(),
   dying(),
+  winning(),
   backflipping(),
   backflip_direction(),
   peekingX(),
   peekingY(),
+  ability_time(),
+  stone(),
   swimming(),
   speedlimit(),
   scripting_controller_old(0),
   jump_early_apex(),
   on_ice(),
   ice_this_frame(),
+  lightsprite(SpriteManager::current()->create("images/creatures/tux/light.sprite")),
+  powersprite(SpriteManager::current()->create("images/creatures/tux/powerups.sprite")),
   dir(),
   old_dir(),
   last_ground_y(),
@@ -134,7 +142,7 @@ Player::Player(PlayerStatus* _player_status, const std::string& name) :
   on_ground_flag(),
   jumping(),
   can_jump(),
-  jump_button_timer(), 
+  jump_button_timer(),
   wants_buttjump(),
   does_buttjump(),
   invincible_timer(),
@@ -142,40 +150,42 @@ Player::Player(PlayerStatus* _player_status, const std::string& name) :
   safe_timer(),
   kick_timer(),
   shooting_timer(),
+  ability_timer(),
+  cooldown_timer(),
   dying_timer(),
   growing(),
   backflip_timer(),
   physic(),
   visible(),
-  grabbed_object(NULL), 
+  grabbed_object(NULL),
   sprite(),
   airarrow(),
   floor_normal(),
-  ghost_mode(false), 
-  edit_mode(false), 
+  ghost_mode(false),
+  edit_mode(false),
   unduck_hurt_timer(),
   idle_timer(),
   idle_stage(0),
   climbing(0)
 {
-  this->name = name;
-  controller = g_jk_controller->get_main_controller();
+  this->name = name_;
+  controller = InputManager::current()->get_controller();
   scripting_controller.reset(new CodeController());
   // if/when we have complete penny gfx, we can
   // load those instead of Tux's sprite in the
   // constructor
-  sprite = sprite_manager->create("images/creatures/tux/tux.sprite");
+  sprite = SpriteManager::current()->create("images/creatures/tux/tux.sprite");
   airarrow = Surface::create("images/engine/hud/airarrow.png");
   idle_timer.start(IDLE_TIME[0]/1000.0f);
 
-  sound_manager->preload("sounds/bigjump.wav");
-  sound_manager->preload("sounds/jump.wav");
-  sound_manager->preload("sounds/hurt.wav");
-  sound_manager->preload("sounds/kill.wav");
-  sound_manager->preload("sounds/skid.wav");
-  sound_manager->preload("sounds/flip.wav");
-  sound_manager->preload("sounds/invincible_start.ogg");
-  sound_manager->preload("sounds/splash.ogg");
+  SoundManager::current()->preload("sounds/bigjump.wav");
+  SoundManager::current()->preload("sounds/jump.wav");
+  SoundManager::current()->preload("sounds/hurt.wav");
+  SoundManager::current()->preload("sounds/kill.wav");
+  SoundManager::current()->preload("sounds/skid.wav");
+  SoundManager::current()->preload("sounds/flip.wav");
+  SoundManager::current()->preload("sounds/invincible_start.ogg");
+  SoundManager::current()->preload("sounds/splash.wav");
 
   init();
 }
@@ -213,11 +223,17 @@ Player::init()
   deactivated = false;
   backflipping = false;
   backflip_direction = 0;
+  sprite->set_angle(0.0f);
+  powersprite->set_angle(0.0f);
+  lightsprite->set_angle(0.0f);
   visible = true;
+  ability_time = 0;
+  stone = false;
   swimming = false;
   on_ice = false;
   ice_this_frame = false;
   speedlimit = 0; //no special limit
+  lightsprite->set_blend(Blend(GL_SRC_ALPHA, GL_ONE));
 
   on_ground_flag = false;
   grabbed_object = NULL;
@@ -258,9 +274,9 @@ Player::set_speedlimit(float newlimit)
 }
 
 void
-Player::set_controller(Controller* controller)
+Player::set_controller(Controller* controller_)
 {
-  this->controller = controller;
+  this->controller = controller_;
 }
 
 void
@@ -272,7 +288,7 @@ Player::set_winning()
   }
 }
 
-void 
+void
 Player::use_scripting_controller(bool use_or_release)
 {
   if ((use_or_release == true) && (controller != scripting_controller.get())) {
@@ -285,7 +301,7 @@ Player::use_scripting_controller(bool use_or_release)
   }
 }
 
-void 
+void
 Player::do_scripting_controller(std::string control, bool pressed)
 {
   for(int i = 0; Controller::controlNames[i] != 0; ++i) {
@@ -323,6 +339,9 @@ Player::trigger_sequence(std::string sequence_name)
   if (climbing) stop_climbing(*climbing);
   backflipping = false;
   backflip_direction = 0;
+  sprite->set_angle(0.0f);
+  powersprite->set_angle(0.0f);
+  lightsprite->set_angle(0.0f);
   GameSession::current()->start_sequence(sequence_name);
 }
 
@@ -335,6 +354,7 @@ Player::update(float elapsed_time)
   no_water = true;
 
   if(dying && dying_timer.check()) {
+    set_bonus(NO_BONUS, true);
     dead = true;
     return;
   }
@@ -342,9 +362,11 @@ Player::update(float elapsed_time)
   if(!dying && !deactivated)
     handle_input();
 
+  /*
   // handle_input() calls apply_friction() when Tux is not walking, so we'll have to do this ourselves
   if (deactivated)
-    apply_friction();
+  apply_friction();
+  */
 
   // extend/shrink tux collision rectangle so that we fall through/walk over 1
   // tile holes
@@ -355,7 +377,7 @@ Player::update(float elapsed_time)
   }
 
   // on downward slopes, adjust vertical velocity so tux walks smoothly down
-  if (on_ground()) {
+  if (on_ground() && !dying) {
     if(floor_normal.y != 0) {
       if ((floor_normal.x * physic.get_velocity_x()) >= 0) {
         physic.set_velocity_y(250);
@@ -364,10 +386,18 @@ Player::update(float elapsed_time)
   }
 
   // handle backflipping
-  if (backflipping) {
+  if (backflipping && !dying) {
     //prevent player from changing direction when backflipping
     dir = (backflip_direction == 1) ? LEFT : RIGHT;
     if (backflip_timer.started()) physic.set_velocity_x(100 * backflip_direction);
+    //rotate sprite during flip
+    sprite->set_angle(sprite->get_angle() + (dir==LEFT?1:-1) * elapsed_time * (360.0f / 0.5f));
+    if (player_status->bonus == EARTH_BONUS || player_status->bonus == AIR_BONUS ||
+        (player_status->bonus == FIRE_BONUS && g_config->christmas_mode)) {
+      powersprite->set_angle(sprite->get_angle());
+      if (player_status->bonus == EARTH_BONUS)
+        lightsprite->set_angle(sprite->get_angle());
+    }
   }
 
   // set fall mode...
@@ -384,14 +414,22 @@ Player::update(float elapsed_time)
   // check if we landed
   if(on_ground()) {
     jumping = false;
-    if (backflipping && (!backflip_timer.started())) {
+    if (backflipping && (backflip_timer.get_timegone() > 0.15f)) {
       backflipping = false;
       backflip_direction = 0;
+      physic.set_velocity_x(0);
+      if (!stone) {
+        sprite->set_angle(0.0f);
+        powersprite->set_angle(0.0f);
+        lightsprite->set_angle(0.0f);
+      }
 
       // if controls are currently deactivated, we take care of standing up ourselves
       if (deactivated)
         do_standup();
     }
+    if (player_status->bonus == AIR_BONUS)
+      ability_time = player_status->max_air_time * GLIDE_TIME_PER_FLOWER;
   }
 
   // calculate movement for this frame
@@ -421,13 +459,15 @@ Player::update(float elapsed_time)
       Vector ppos = Vector(px, py);
       Vector pspeed = Vector(0, 0);
       Vector paccel = Vector(0, 0);
-      Sector::current()->add_object(new SpriteParticle("images/objects/particles/sparkle.sprite", 
-                                                       // draw bright sparkle when there is lots of time left, dark sparkle when invincibility is about to end
-                                                       (invincible_timer.get_timeleft() > TUX_INVINCIBLE_TIME_WARNING) ?
-                                                       // make every other a longer sparkle to make trail a bit fuzzy
-                                                       (size_t(game_time*20)%2) ? "small" : "medium"
-                                                       :
-                                                       "dark", ppos, ANCHOR_MIDDLE, pspeed, paccel, LAYER_OBJECTS+1+5));
+      Sector::current()->add_object(std::make_shared<SpriteParticle>(
+                                      "images/objects/particles/sparkle.sprite",
+                                      // draw bright sparkle when there is lots of time left,
+                                      // dark sparkle when invincibility is about to end
+                                      (invincible_timer.get_timeleft() > TUX_INVINCIBLE_TIME_WARNING) ?
+                                      // make every other a longer sparkle to make trail a bit fuzzy
+                                      (size_t(game_time*20)%2) ? "small" : "medium"
+                                      :
+                                      "dark", ppos, ANCHOR_MIDDLE, pspeed, paccel, LAYER_OBJECTS+1+5));
     }
   }
 
@@ -435,6 +475,14 @@ Player::update(float elapsed_time)
     if (sprite->animation_done()) growing = false;
   }
 
+  // when climbing animate only while moving
+  if(climbing){
+    if((physic.get_velocity_x()==0)&&(physic.get_velocity_y()==0))
+      sprite->stop_animation();
+    else
+      sprite->set_animation_loops(-1);
+  }
+
 }
 
 bool
@@ -508,11 +556,11 @@ Player::handle_horizontal_input()
       ax = dirsign * RUN_ACCELERATION_X;
     }
     // limit speed
-    if(vx >= MAX_RUN_XM && dirsign > 0) {
-      vx = MAX_RUN_XM;
+    if(vx >= MAX_RUN_XM + BONUS_RUN_XM *((player_status->bonus == AIR_BONUS) ? 1 : 0) && dirsign > 0) {
+      vx = MAX_RUN_XM + BONUS_RUN_XM *((player_status->bonus == AIR_BONUS) ? 1 : 0);
       ax = 0;
-    } else if(vx <= -MAX_RUN_XM && dirsign < 0) {
-      vx = -MAX_RUN_XM;
+    } else if(vx <= -MAX_RUN_XM - BONUS_RUN_XM *((player_status->bonus == AIR_BONUS) ? 1 : 0) && dirsign < 0) {
+      vx = -MAX_RUN_XM - BONUS_RUN_XM *((player_status->bonus == AIR_BONUS) ? 1 : 0);
       ax = 0;
     }
   }
@@ -533,14 +581,13 @@ Player::handle_horizontal_input()
     // let's skid!
     if(fabs(vx)>SKID_XM && !skidding_timer.started()) {
       skidding_timer.start(SKID_TIME);
-      sound_manager->play("sounds/skid.wav");
+      SoundManager::current()->play("sounds/skid.wav");
       // dust some particles
       Sector::current()->add_object(
-        new Particles(
-          Vector(dir == RIGHT ? get_bbox().p2.x : get_bbox().p1.x, get_bbox().p2.y),
-          dir == RIGHT ? 270+20 : 90-40, dir == RIGHT ? 270+40 : 90-20,
-          Vector(280, -260), Vector(0, 300), 3, Color(.4f, .4f, .4f), 3, .8f,
-          LAYER_OBJECTS+1));
+        std::make_shared<Particles>(
+          Vector(dir == LEFT ? get_bbox().p2.x : get_bbox().p1.x, get_bbox().p2.y),
+          dir == LEFT ? 50 : -70, dir == LEFT ? 70 : -50, 260, 280,
+          Vector(0, 300), 3, Color(.4f, .4f, .4f), 3, .8f, LAYER_OBJECTS+1));
 
       ax *= 2.5;
     } else {
@@ -601,6 +648,8 @@ Player::do_standup() {
     return;
   if (backflipping)
     return;
+  if (stone)
+    return;
 
   if (adjust_height(BIG_TUX_HEIGHT)) {
     duck = false;
@@ -626,9 +675,9 @@ Player::do_backflip() {
 
   backflip_direction = (dir == LEFT)?(+1):(-1);
   backflipping = true;
-  do_jump(-580);
-  sound_manager->play("sounds/flip.wav");
-  backflip_timer.start(0.15f);
+  do_jump((player_status->bonus == AIR_BONUS) ? -720 : -580);
+  SoundManager::current()->play("sounds/flip.wav");
+  backflip_timer.start(TUX_BACKFLIP_TIME);
 }
 
 void
@@ -644,14 +693,14 @@ Player::do_jump(float yspeed) {
 
   // play sound
   if (is_big()) {
-    sound_manager->play("sounds/bigjump.wav");
+    SoundManager::current()->play("sounds/bigjump.wav");
   } else {
-    sound_manager->play("sounds/jump.wav");
+    SoundManager::current()->play("sounds/jump.wav");
   }
 }
 
 void
-Player::early_jump_apex() 
+Player::early_jump_apex()
 {
   if (!jump_early_apex)
   {
@@ -679,27 +728,51 @@ Player::handle_vertical_input()
     jump_button_timer.stop();
     if (duck) {
       // when running, only jump a little bit; else do a backflip
-      if ((physic.get_velocity_x() != 0) || 
-          (controller->hold(Controller::LEFT)) || 
-          (controller->hold(Controller::RIGHT))) 
+      if ((physic.get_velocity_x() != 0) ||
+          (controller->hold(Controller::LEFT)) ||
+          (controller->hold(Controller::RIGHT)))
       {
         do_jump(-300);
       }
-      else 
+      else
       {
         do_backflip();
       }
     } else {
+      // airflower allows for higher jumps-
       // jump a bit higher if we are running; else do a normal jump
-      if (fabs(physic.get_velocity_x()) > MAX_WALK_XM) do_jump(-580); else do_jump(-520);
+      if(player_status->bonus == AIR_BONUS)
+        do_jump((fabs(physic.get_velocity_x()) > MAX_WALK_XM) ? -620 : -580);
+      else
+        do_jump((fabs(physic.get_velocity_x()) > MAX_WALK_XM) ? -580 : -520);
     }
-  }
+    // airflower glide only when holding jump key
+  } else  if (controller->hold(Controller::JUMP) && player_status->bonus == AIR_BONUS && physic.get_velocity_y() > MAX_GLIDE_YM) {
+      if (ability_time > 0 && !ability_timer.started())
+        ability_timer.start(ability_time);
+      else if (ability_timer.started()) {
+        // glide stops after some duration or if buttjump is initiated
+        if ((ability_timer.get_timeleft() <= 0.05f) || controller->hold(Controller::DOWN)) {
+          ability_time = 0;
+          ability_timer.stop();
+        } else {
+          physic.set_velocity_y(MAX_GLIDE_YM);
+          physic.set_acceleration_y(0);
+        }
+      }
+    }
+
+
   // Let go of jump key
   else if(!controller->hold(Controller::JUMP)) {
     if (!backflipping && jumping && physic.get_velocity_y() < 0) {
       jumping = false;
       early_jump_apex();
     }
+    if (player_status->bonus == AIR_BONUS && ability_timer.started()){
+      ability_time = ability_timer.get_timeleft();
+      ability_timer.stop();
+    }
   }
 
   if(jump_early_apex && physic.get_velocity_y() >= 0) {
@@ -764,27 +837,65 @@ Player::handle_input()
   }
 
   /* Handle horizontal movement: */
-  if (!backflipping) handle_horizontal_input();
+  if (!backflipping && !stone) handle_horizontal_input();
 
   /* Jump/jumping? */
   if (on_ground())
     can_jump = true;
 
   /* Handle vertical movement: */
-  handle_vertical_input();
+  if (!stone) handle_vertical_input();
 
   /* Shoot! */
   if (controller->pressed(Controller::ACTION) && (player_status->bonus == FIRE_BONUS || player_status->bonus == ICE_BONUS)) {
-    if(Sector::current()->add_bullet(
-         get_pos() + ((dir == LEFT)? Vector(0, bbox.get_height()/2)
-                      : Vector(32, bbox.get_height()/2)),
-         player_status,
-         physic.get_velocity_x(), dir))
+    if((player_status->bonus == FIRE_BONUS &&
+      Sector::current()->get_active_bullets() < player_status->max_fire_bullets) ||
+      (player_status->bonus == ICE_BONUS &&
+      Sector::current()->get_active_bullets() < player_status->max_ice_bullets))
+    {
+      Vector pos = get_pos() + ((dir == LEFT)? Vector(0, bbox.get_height()/2) : Vector(32, bbox.get_height()/2));
+      auto new_bullet = std::make_shared<Bullet>(pos, physic.get_velocity_x(), dir, player_status->bonus);
+      Sector::current()->add_object(new_bullet);
+
+      SoundManager::current()->play("sounds/shoot.wav");
       shooting_timer.start(SHOOTING_TIME);
+    }
+  }
+
+  /* Turn to Stone */
+  if (controller->pressed(Controller::DOWN) && player_status->bonus == EARTH_BONUS && !cooldown_timer.started()) {
+    if (controller->hold(Controller::ACTION) && !ability_timer.started()) {
+      ability_timer.start(player_status->max_earth_time * STONE_TIME_PER_FLOWER);
+      powersprite->stop_animation();
+      stone = true;
+      physic.set_gravity_modifier(1.0f); // Undo jump_early_apex
+    }
+  }
+
+  if (stone)
+    apply_friction();
+
+  /* Revert from Stone */
+  if (stone && (!controller->hold(Controller::ACTION) || ability_timer.get_timeleft() <= 0.5f)) {
+    cooldown_timer.start(ability_timer.get_timegone()/2.0f); //The longer stone form is used, the longer until it can be used again
+    ability_timer.stop();
+    sprite->set_angle(0.0f);
+    powersprite->set_angle(0.0f);
+    lightsprite->set_angle(0.0f);
+    stone = false;
+    for (int i = 0; i < 8; i++)
+    {
+      Vector ppos = Vector(get_bbox().get_left() + 8 + 16*((int)i/4), get_bbox().get_top() + 16*(i%4));
+      float grey = graphicsRandom.randf(.4f, .8f);
+      Color pcolor = Color(grey, grey, grey);
+      Sector::current()->add_object(std::make_shared<Particles>(ppos, -60, 240, 42, 81, Vector(0, 500),
+                                                                8, pcolor, 4 + graphicsRandom.randf(-0.4, 0.4),
+                                                                0.8 + graphicsRandom.randf(0, 0.4), LAYER_OBJECTS+2));
+    }
   }
 
   /* Duck or Standup! */
-  if (controller->hold(Controller::DOWN)) {
+  if (controller->hold(Controller::DOWN) && !stone) {
     do_duck();
   } else {
     do_standup();
@@ -798,18 +909,18 @@ Player::handle_input()
     if(moving_object) {
       // move the grabbed object a bit away from tux
       Rectf grabbed_bbox = moving_object->get_bbox();
-      Rectf dest;
-      dest.p2.y = bbox.get_top() + bbox.get_height()*0.66666;
-      dest.p1.y = dest.p2.y - grabbed_bbox.get_height();
+      Rectf dest_;
+      dest_.p2.y = bbox.get_top() + bbox.get_height()*0.66666;
+      dest_.p1.y = dest_.p2.y - grabbed_bbox.get_height();
       if(dir == LEFT) {
-        dest.p2.x = bbox.get_left() - 1;
-        dest.p1.x = dest.p2.x - grabbed_bbox.get_width();
+        dest_.p2.x = bbox.get_left() - 1;
+        dest_.p1.x = dest_.p2.x - grabbed_bbox.get_width();
       } else {
-        dest.p1.x = bbox.get_right() + 1;
-        dest.p2.x = dest.p1.x + grabbed_bbox.get_width();
+        dest_.p1.x = bbox.get_right() + 1;
+        dest_.p2.x = dest_.p1.x + grabbed_bbox.get_width();
       }
-      if(Sector::current()->is_free_of_movingstatics(dest)) {
-        moving_object->set_pos(dest.p1);
+      if(Sector::current()->is_free_of_tiles(dest_, true)) {
+        moving_object->set_pos(dest_.p1);
         if(controller->hold(Controller::UP)) {
           grabbed_object->ungrab(*this, UP);
         } else {
@@ -821,6 +932,15 @@ Player::handle_input()
       log_debug << "Non MovingObject grabbed?!?" << std::endl;
     }
   }
+
+  /* stop backflipping at will */
+  if( backflipping && ( !controller->hold(Controller::JUMP) && !backflip_timer.started()) ){
+    backflipping = false;
+    backflip_direction = 0;
+    sprite->set_angle(0.0f);
+    powersprite->set_angle(0.0f);
+    lightsprite->set_angle(0.0f);
+  }
 }
 
 void
@@ -863,8 +983,6 @@ Player::try_grab()
       // make sure the Portable is a MovingObject
       MovingObject* moving_object = dynamic_cast<MovingObject*> (portable);
       assert(moving_object);
-      if(moving_object == NULL)
-        continue;
 
       // make sure the Portable isn't currently non-solid
       if(moving_object->get_group() == COLGROUP_DISABLED) continue;
@@ -929,6 +1047,10 @@ Player::add_bonus(const std::string& bonustype)
     type = FIRE_BONUS;
   } else if(bonustype == "iceflower") {
     type = ICE_BONUS;
+  } else if(bonustype == "airflower") {
+    type = AIR_BONUS;
+  } else if(bonustype == "earthflower") {
+    type = EARTH_BONUS;
   } else if(bonustype == "none") {
     type = NO_BONUS;
   } else {
@@ -950,11 +1072,7 @@ Player::add_bonus(BonusType type, bool animate)
 
   // ignore GROWUP_BONUS if we're already big
   if (type == GROWUP_BONUS) {
-    if (player_status->bonus == GROWUP_BONUS)
-      return true;
-    if (player_status->bonus == FIRE_BONUS)
-      return true;
-    if (player_status->bonus == ICE_BONUS)
+    if (player_status->bonus != NO_BONUS)
       return true;
   }
 
@@ -966,7 +1084,7 @@ Player::set_bonus(BonusType type, bool animate)
 {
   if((player_status->bonus == NO_BONUS) && (type != NO_BONUS)) {
     if (!adjust_height(BIG_TUX_HEIGHT)) {
-      printf("can't adjust\n");
+      log_debug << "Can't adjust Tux height" << std::endl;
       return false;
     }
     if(animate) {
@@ -987,7 +1105,12 @@ Player::set_bonus(BonusType type, bool animate)
       Vector pspeed = Vector(((dir==LEFT) ? +100 : -100), -300);
       Vector paccel = Vector(0, 1000);
       std::string action = (dir==LEFT)?"left":"right";
-      Sector::current()->add_object(new SpriteParticle("images/objects/particles/firetux-helmet.sprite", action, ppos, ANCHOR_TOP, pspeed, paccel, LAYER_OBJECTS-1));
+      if (g_config->christmas_mode) {
+        Sector::current()->add_object(std::make_shared<SpriteParticle>("images/objects/particles/santatux-hat.sprite", action, ppos,ANCHOR_TOP, pspeed, paccel, LAYER_OBJECTS-1)); 
+      }
+      else {
+        Sector::current()->add_object(std::make_shared<SpriteParticle>("images/objects/particles/firetux-helmet.sprite", action, ppos, ANCHOR_TOP, pspeed, paccel, LAYER_OBJECTS-1));
+      }
       if (climbing) stop_climbing(*climbing);
     }
     if ((player_status->bonus == ICE_BONUS) && (animate)) {
@@ -996,24 +1119,46 @@ Player::set_bonus(BonusType type, bool animate)
       Vector pspeed = Vector(((dir==LEFT) ? +100 : -100), -300);
       Vector paccel = Vector(0, 1000);
       std::string action = (dir==LEFT)?"left":"right";
-      Sector::current()->add_object(new SpriteParticle("images/objects/particles/icetux-cap.sprite", action, ppos, ANCHOR_TOP, pspeed, paccel, LAYER_OBJECTS-1));
+      Sector::current()->add_object(std::make_shared<SpriteParticle>("images/objects/particles/icetux-cap.sprite", action, ppos, ANCHOR_TOP, pspeed, paccel, LAYER_OBJECTS-1));
+      if (climbing) stop_climbing(*climbing);
+    }
+    if ((player_status->bonus == AIR_BONUS) && (animate)) {
+      // visually lose hat
+      Vector ppos = Vector((bbox.p1.x + bbox.p2.x) / 2, bbox.p1.y);
+      Vector pspeed = Vector(((dir==LEFT) ? +100 : -100), -300);
+      Vector paccel = Vector(0, 1000);
+      std::string action = (dir==LEFT)?"left":"right";
+      Sector::current()->add_object(std::make_shared<SpriteParticle>("images/objects/particles/airtux-hat.sprite", action, ppos, ANCHOR_TOP, pspeed, paccel, LAYER_OBJECTS-1));
+      if (climbing) stop_climbing(*climbing);
+    }
+    if ((player_status->bonus == EARTH_BONUS) && (animate)) {
+      // visually lose hard-hat
+      Vector ppos = Vector((bbox.p1.x + bbox.p2.x) / 2, bbox.p1.y);
+      Vector pspeed = Vector(((dir==LEFT) ? +100 : -100), -300);
+      Vector paccel = Vector(0, 1000);
+      std::string action = (dir==LEFT)?"left":"right";
+      Sector::current()->add_object(std::make_shared<SpriteParticle>("images/objects/particles/earthtux-hardhat.sprite", action, ppos, ANCHOR_TOP, pspeed, paccel, LAYER_OBJECTS-1));
       if (climbing) stop_climbing(*climbing);
     }
     player_status->max_fire_bullets = 0;
     player_status->max_ice_bullets = 0;
+    player_status->max_air_time = 0;
+    player_status->max_earth_time = 0;
   }
   if (type == FIRE_BONUS) player_status->max_fire_bullets++;
   if (type == ICE_BONUS) player_status->max_ice_bullets++;
+  if (type == AIR_BONUS) player_status->max_air_time++;
+  if (type == EARTH_BONUS) player_status->max_earth_time++;
 
   player_status->bonus = type;
   return true;
 }
 
 void
-Player::set_visible(bool visible)
+Player::set_visible(bool visible_)
 {
-  this->visible = visible;
-  if( visible )
+  this->visible = visible_;
+  if( visible_ )
     set_group(COLGROUP_MOVING);
   else
     set_group(COLGROUP_DISABLED);
@@ -1051,9 +1196,16 @@ Player::draw(DrawingContext& context)
   if (player_status->bonus == GROWUP_BONUS)
     sa_prefix = "big";
   else if (player_status->bonus == FIRE_BONUS)
-    sa_prefix = "fire";
+    if(g_config->christmas_mode)
+      sa_prefix = "santa";
+    else
+      sa_prefix = "fire";
   else if (player_status->bonus == ICE_BONUS)
     sa_prefix = "ice";
+  else if (player_status->bonus == AIR_BONUS)
+    sa_prefix = "air";
+  else if (player_status->bonus == EARTH_BONUS)
+    sa_prefix = "earth";
   else
     sa_prefix = "small";
 
@@ -1072,6 +1224,9 @@ Player::draw(DrawingContext& context)
     // do_duck() will take care of cancelling growing manually
     // update() will take care of cancelling when growing completed
   }
+  else if (stone) {
+    sprite->set_action(sprite->get_action()+"-stone");
+  }
   else if (climbing) {
     sprite->set_action(sa_prefix+"-climbing"+sa_postfix);
   }
@@ -1090,8 +1245,10 @@ Player::draw(DrawingContext& context)
   else if ((wants_buttjump || does_buttjump) && is_big()) {
     sprite->set_action(sa_prefix+"-buttjump"+sa_postfix);
   }
-  else if (!on_ground()) {
-    sprite->set_action(sa_prefix+"-jump"+sa_postfix);
+  else if (!on_ground() || fall_mode != ON_GROUND) {
+    if(physic.get_velocity_x() != 0 || fall_mode != ON_GROUND) {
+        sprite->set_action(sa_prefix+"-jump"+sa_postfix);
+    }
   }
   else {
     if (fabsf(physic.get_velocity_x()) < 1.0f) {
@@ -1123,6 +1280,16 @@ Player::draw(DrawingContext& context)
     }
   }
 
+  /* Set Tux powerup sprite action */
+  if (player_status->bonus == EARTH_BONUS) {
+    powersprite->set_action(sprite->get_action());
+    lightsprite->set_action(sprite->get_action());
+  } else if (player_status->bonus == AIR_BONUS) {
+    powersprite->set_action(sprite->get_action());
+  } else if (player_status->bonus == FIRE_BONUS && g_config->christmas_mode) {
+    powersprite->set_action(sprite->get_action());
+  }
+
   /*
   // Tux is holding something
   if ((grabbed_object != 0 && physic.get_velocity_y() == 0) ||
@@ -1136,8 +1303,38 @@ Player::draw(DrawingContext& context)
   /* Draw Tux */
   if (safe_timer.started() && size_t(game_time*40)%2)
     ;  // don't draw Tux
+  else if (player_status->bonus == EARTH_BONUS){ // draw special effects with earthflower bonus
+    // shake at end of maximum stone duration
+    Vector shake_delta = (stone && ability_timer.get_timeleft() < 1.0f) ? Vector(graphicsRandom.rand(-3,3), 0) : Vector(0,0);
+    sprite->draw(context, get_pos() + shake_delta, LAYER_OBJECTS + 1);
+    // draw hardhat
+    powersprite->draw(context, get_pos() + shake_delta, LAYER_OBJECTS + 1);
+    // light
+    context.push_target();
+    context.set_target(DrawingContext::LIGHTMAP);
+    lightsprite->draw(context, get_pos(), 0);
+    context.pop_target();
+    // give an indicator that stone form cannot be used for a while
+    if (cooldown_timer.started() && graphicsRandom.rand(0, 4) == 0) {
+      float px = graphicsRandom.randf(bbox.p1.x, bbox.p2.x);
+      float py = bbox.p2.y+8;
+      Vector ppos = Vector(px, py);
+      Sector::current()->add_object(std::make_shared<SpriteParticle>(
+        "images/objects/particles/sparkle.sprite", "dark",
+        ppos, ANCHOR_MIDDLE, Vector(0, 0), Vector(0, 0), LAYER_OBJECTS+1+5));
+    }
+  }
   else {
-    sprite->draw(context, get_pos(), LAYER_OBJECTS + 1);
+    if(dying)
+      sprite->draw(context, get_pos(), Sector::current()->get_foremost_layer() + 1);
+    else
+      sprite->draw(context, get_pos(), LAYER_OBJECTS + 1);
+
+    if (player_status->bonus == AIR_BONUS)
+      powersprite->draw(context, get_pos(), LAYER_OBJECTS + 1);
+    else if(player_status->bonus == FIRE_BONUS && g_config->christmas_mode) {
+      powersprite->draw(context, get_pos(), LAYER_OBJECTS + 1);
+    }
   }
 
 }
@@ -1159,7 +1356,7 @@ Player::collision_tile(uint32_t tile_attributes)
     if( tile_attributes & Tile::WATER ){
       swimming = true;
       no_water = false;
-      sound_manager->play( "sounds/splash.ogg" );
+      SoundManager::current()->play( "sounds/splash.wav" );
     }
   }
 #endif
@@ -1180,21 +1377,19 @@ Player::collision_solid(const CollisionHit& hit)
     on_ground_flag = true;
     floor_normal = hit.slope_normal;
 
-    // Butt Jump landed    
+    // Butt Jump landed
     if (does_buttjump) {
       does_buttjump = false;
       physic.set_velocity_y(-300);
       on_ground_flag = false;
-      Sector::current()->add_object(new Particles(
+      Sector::current()->add_object(std::make_shared<Particles>(
                                       Vector(get_bbox().p2.x, get_bbox().p2.y),
-                                      270+20, 270+40,
-                                      Vector(280, -260), Vector(0, 300), 3, Color(.4f, .4f, .4f), 3, .8f,
-                                      LAYER_OBJECTS+1));
-      Sector::current()->add_object(new Particles(
+                                      50, 70, 260, 280, Vector(0, 300), 3,
+                                      Color(.4f, .4f, .4f), 3, .8f, LAYER_OBJECTS+1));
+      Sector::current()->add_object(std::make_shared<Particles>(
                                       Vector(get_bbox().p1.x, get_bbox().p2.y),
-                                      90-40, 90-20,
-                                      Vector(280, -260), Vector(0, 300), 3, Color(.4f, .4f, .4f), 3, .8f,
-                                      LAYER_OBJECTS+1));
+                                      -70, -50, 260, 280, Vector(0, 300), 3,
+                                      Color(.4f, .4f, .4f), 3, .8f, LAYER_OBJECTS+1));
       Sector::current()->camera->shake(.1f, 0, 5);
     }
 
@@ -1249,6 +1444,8 @@ Player::collision(GameObject& other, const CollisionHit& hit)
   if(badguy != NULL) {
     if(safe_timer.started() || invincible_timer.started())
       return FORCE_MOVE;
+    if(stone)
+      return ABORT_MOVE;
 
     return CONTINUE;
   }
@@ -1259,7 +1456,7 @@ Player::collision(GameObject& other, const CollisionHit& hit)
 void
 Player::make_invincible()
 {
-  sound_manager->play("sounds/invincible_start.ogg");
+  SoundManager::current()->play("sounds/invincible_start.ogg");
   invincible_timer.start(TUX_INVINCIBLE_TIME);
   Sector::current()->play_music(HERRING_MUSIC);
 }
@@ -1271,7 +1468,7 @@ Player::kill(bool completely)
   if(dying || deactivated || is_winning() )
     return;
 
-  if(!completely && (safe_timer.started() || invincible_timer.started()))
+  if(!completely && (safe_timer.started() || invincible_timer.started() || stone))
     return;
 
   growing = false;
@@ -1280,11 +1477,17 @@ Player::kill(bool completely)
 
   physic.set_velocity_x(0);
 
+  sprite->set_angle(0.0f);
+  powersprite->set_angle(0.0f);
+  lightsprite->set_angle(0.0f);
+
   if(!completely && is_big()) {
-    sound_manager->play("sounds/hurt.wav");
+    SoundManager::current()->play("sounds/hurt.wav");
 
     if(player_status->bonus == FIRE_BONUS
-       || player_status->bonus == ICE_BONUS) {
+      || player_status->bonus == ICE_BONUS
+      || player_status->bonus == AIR_BONUS
+      || player_status->bonus == EARTH_BONUS) {
       safe_timer.start(TUX_SAFE_TIME);
       set_bonus(GROWUP_BONUS, true);
     } else if(player_status->bonus == GROWUP_BONUS) {
@@ -1292,6 +1495,9 @@ Player::kill(bool completely)
       adjust_height(SMALL_TUX_HEIGHT);
       duck = false;
       backflipping = false;
+      sprite->set_angle(0.0f);
+      powersprite->set_angle(0.0f);
+      lightsprite->set_angle(0.0f);
       set_bonus(NO_BONUS, true);
     } else if(player_status->bonus == NO_BONUS) {
       safe_timer.start(TUX_SAFE_TIME);
@@ -1299,7 +1505,7 @@ Player::kill(bool completely)
       duck = false;
     }
   } else {
-    sound_manager->play("sounds/kill.wav");
+    SoundManager::current()->play("sounds/kill.wav");
 
     // do not die when in edit mode
     if (edit_mode) {
@@ -1312,7 +1518,7 @@ Player::kill(bool completely)
       for (int i = 0; i < 5; i++)
       {
         // the numbers: starting x, starting y, velocity y
-        Sector::current()->add_object(new FallingCoin(get_pos() +
+        Sector::current()->add_object(std::make_shared<FallingCoin>(get_pos() +
                                                       Vector(graphicsRandom.rand(5), graphicsRandom.rand(-32,18)),
                                                       graphicsRandom.rand(-100,100)));
       }
@@ -1335,7 +1541,7 @@ Player::kill(bool completely)
 
     // TODO: need nice way to handle players dying in co-op mode
     Sector::current()->effect->fade_out(3.0);
-    sound_manager->stop_music(3.0);
+    SoundManager::current()->pause_music(3.0);
   }
 }
 
@@ -1351,6 +1557,9 @@ Player::move(const Vector& vector)
     set_size(TUX_WIDTH, SMALL_TUX_HEIGHT);
   duck = false;
   backflipping = false;
+  sprite->set_angle(0.0f);
+  powersprite->set_angle(0.0f);
+  lightsprite->set_angle(0.0f);
   last_ground_y = vector.y;
   if (climbing) stop_climbing(*climbing);
 
@@ -1399,7 +1608,7 @@ Player::add_velocity(const Vector& velocity, const Vector& end_speed)
     physic.set_velocity_y(std::max(physic.get_velocity_y() + velocity.y, end_speed.y));
 }
 
-Vector 
+Vector
 Player::get_velocity()
 {
   return physic.get_velocity();
@@ -1408,10 +1617,12 @@ Player::get_velocity()
 void
 Player::bounce(BadGuy& )
 {
-  if(controller->hold(Controller::JUMP))
-    physic.set_velocity_y(-520);
-  else
-    physic.set_velocity_y(-300);
+  if(!(player_status->bonus == AIR_BONUS))
+    physic.set_velocity_y(controller->hold(Controller::JUMP) ? -520 : -300);
+  else {
+    physic.set_velocity_y(controller->hold(Controller::JUMP) ? -580 : -340);
+    ability_time = player_status->max_air_time * GLIDE_TIME_PER_FLOWER;
+  }
 }
 
 //scripting Functions Below
@@ -1442,6 +1653,11 @@ void Player::walk(float speed)
   physic.set_velocity_x(speed);
 }
 
+void Player::set_dir(bool right)
+{
+  dir = right ? RIGHT : LEFT;
+}
+
 void
 Player::set_ghost_mode(bool enable)
 {
@@ -1469,7 +1685,7 @@ Player::set_edit_mode(bool enable)
   edit_mode = enable;
 }
 
-void 
+void
 Player::start_climbing(Climbable& climbable)
 {
   if (climbing || !&climbable) return;
@@ -1481,17 +1697,20 @@ Player::start_climbing(Climbable& climbable)
   if (backflipping) {
     backflipping = false;
     backflip_direction = 0;
+    sprite->set_angle(0.0f);
+    powersprite->set_angle(0.0f);
+    lightsprite->set_angle(0.0f);
   }
 }
 
-void 
+void
 Player::stop_climbing(Climbable& /*climbable*/)
 {
   if (!climbing) return;
 
   climbing = 0;
 
-  if (grabbed_object) {    
+  if (grabbed_object) {
     grabbed_object->ungrab(*this, dir);
     grabbed_object = NULL;
   }
@@ -1535,7 +1754,7 @@ Player::handle_input_climbing()
     if (can_jump) {
       stop_climbing(*climbing);
       return;
-    }  
+    }
   } else {
     can_jump = true;
   }
