@@ -27,9 +27,9 @@ ScriptedObject::ScriptedObject(const Reader& lisp) :
   MovingSprite(lisp, LAYER_OBJECTS, COLGROUP_MOVING_STATIC),
   physic(),
   name(),
-  solid(true), 
-  physic_enabled(true), 
-  visible(true), 
+  solid(true),
+  physic_enabled(true),
+  visible(true),
   new_vel_set(false),
   new_vel()
 {
@@ -78,8 +78,8 @@ ScriptedObject::move(float x, float y)
 void
 ScriptedObject::set_pos(float x, float y)
 {
-  printf("SetPos: %f %f\n", x, y);
-  bbox.set_pos(Vector(x, y));
+  // printf("SetPos: %f %f\n", x, y);
+  MovingObject::set_pos(Vector(x, y));
   physic.reset();
 }
 
@@ -115,9 +115,9 @@ ScriptedObject::get_velocity_y()
 }
 
 void
-ScriptedObject::set_visible(bool visible)
+ScriptedObject::set_visible(bool visible_)
 {
-  this->visible = visible;
+  this->visible = visible_;
 }
 
 bool
@@ -127,9 +127,9 @@ ScriptedObject::is_visible()
 }
 
 void
-ScriptedObject::set_solid(bool solid)
+ScriptedObject::set_solid(bool solid_)
 {
-  this->solid = solid;
+  this->solid = solid_;
   if( solid ){
     set_group( COLGROUP_MOVING_STATIC );
   } else {
