@@ -1,5 +1,5 @@
 //  SuperTux
-//  Copyright (C) 2009 Ingo Ruhnke <grumbel@gmx.de>
+//  Copyright (C) 2009 Ingo Ruhnke <grumbel@gmail.com>
 //
 //  This program is free software: you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -19,22 +19,31 @@
 
 #include "gui/menu.hpp"
 
-enum {
-  ADDON_LIST_START_ID = 10
-};
-
 class Addon;
+class AddonManager;
 
 class AddonMenu : public Menu
 {
 private:
-  std::vector<Addon*> m_addons;
+  enum {
+    MNID_CHECK_ONLINE,
+    MNID_NOTHING_NEW,
+    MNID_ADDON_LIST_START = 10
+  };
+
+private:
+  AddonManager& m_addon_manager;
+  std::vector<std::string> m_installed_addons;
+  std::vector<std::string> m_repository_addons;
 
 public:
   AddonMenu();
 
   void refresh();
-  void check_menu();
+  void menu_action(MenuItem* item) override;
+
+private:
+  void rebuild_menu();
 
 private:
   AddonMenu(const AddonMenu&);
