@@ -1,5 +1,5 @@
 //  SuperTux
-//  Copyright (C) 2009 Ingo Ruhnke <grumbel@gmx.de>
+//  Copyright (C) 2009 Ingo Ruhnke <grumbel@gmail.com>
 //
 //  This program is free software: you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -16,24 +16,42 @@
 
 #include "supertux/menu/game_menu.hpp"
 
+#include "gui/menu.hpp"
+#include "gui/menu_item.hpp"
+#include "gui/menu_manager.hpp"
+#include "supertux/game_session.hpp"
 #include "supertux/level.hpp"
 #include "supertux/menu/menu_storage.hpp"
 #include "supertux/menu/options_menu.hpp"
+#include "supertux/screen_manager.hpp"
 #include "util/gettext.hpp"
 
-GameMenu::GameMenu(const Level& level)
+GameMenu::GameMenu()
 {
-  add_label(level.name);
+  Level* level = GameSession::current()->get_current_level();
+
+  add_label(level->name);
   add_hl();
   add_entry(MNID_CONTINUE, _("Continue"));
-  add_submenu(_("Options"), MenuStorage::get_options_menu());
+  add_submenu(_("Options"), MenuStorage::INGAME_OPTIONS_MENU);
   add_hl();
   add_entry(MNID_ABORTLEVEL, _("Abort Level"));
 }
 
 void
-GameMenu::check_menu()
+GameMenu::menu_action(MenuItem* item)
 {
+  switch (item->id)
+  {
+    case MNID_CONTINUE:
+      MenuManager::instance().clear_menu_stack();
+      GameSession::current()->toggle_pause();
+      break;
+
+    case MNID_ABORTLEVEL:
+      GameSession::current()->abort_level();
+      break;
+  }
 }
 
 /* EOF */
