@@ -1,5 +1,5 @@
 //  SuperTux
-//  Copyright (C) 2009 Ingo Ruhnke <grumbel@gmx.de>
+//  Copyright (C) 2009 Ingo Ruhnke <grumbel@gmail.com>
 //
 //  This program is free software: you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -30,9 +30,9 @@ class GameMenu : public Menu
 {
 private:
 public:
-  GameMenu(const Level& level);
+  GameMenu();
 
-  void check_menu();
+  void menu_action(MenuItem* item) override;
 
 private:
   GameMenu(const GameMenu&);
