@@ -1,5 +1,5 @@
 //  SuperTux
-//  Copyright (C) 2009 Ingo Ruhnke <grumbel@gmx.de>
+//  Copyright (C) 2009 Ingo Ruhnke <grumbel@gmail.com>
 //
 //  This program is free software: you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -35,15 +35,11 @@ enum MainMenuIDs {
 
 class MainMenu : public Menu
 {
-private:
-  std::auto_ptr<AddonMenu>   m_addon_menu;
-  std::auto_ptr<ContribMenu> m_contrib_menu;
-  std::auto_ptr<World>       m_main_world;
-
 public:
   MainMenu();
 
-  void check_menu();
+  void on_window_resize() override;
+  void menu_action(MenuItem* item) override;
 
 private:
   MainMenu(const MainMenu&);
