@@ -30,36 +30,35 @@
 
 #include <algorithm>
 #include <sstream>
+#include <stdio.h>
 
 enum OptionsMenuIDs {
   MNID_FULLSCREEN,
   MNID_FULLSCREEN_RESOLUTION,
   MNID_MAGNIFICATION,
   MNID_ASPECTRATIO,
-  MNID_PROFILES,
   MNID_SOUND,
-  MNID_MUSIC
+  MNID_MUSIC,
+  MNID_DEVELOPER_MODE,
+  MNID_CHRISTMAS_MODE
 };
 
-OptionsMenu::OptionsMenu() :
-  language_menu()
+OptionsMenu::OptionsMenu(bool complete)
 {
-  language_menu.reset(new LanguageMenu());
-
   add_label(_("Options"));
   add_hl();
 
-  // Language change should only be possible in the main menu, since elsewhere it might not always work fully
-  // FIXME: Implement me: if (get_parent() == main_menu)
-  add_submenu(_("Select Language"), language_menu.get())
-    ->set_help(_("Select a different language to display text in"));
+  if (complete)
+  {
+    // Language and profile changes are only be possible in the
+    // main menu, since elsewhere it might not always work fully
+    add_submenu(_("Select Language"), MenuStorage::LANGUAGE_MENU)
+      ->set_help(_("Select a different language to display text in"));
 
-  add_submenu(_("Select Profile"), MenuStorage::get_profile_menu())
-    ->set_help(_("Select a profile to play with"));
+    add_submenu(_("Select Profile"), MenuStorage::PROFILE_MENU)
+      ->set_help(_("Select a profile to play with"));
+  }
 
-  add_toggle(MNID_PROFILES, _("Profile on Startup"), g_config->sound_enabled)
-    ->set_help(_("Select your profile immediately after start-up"));
-  
   add_toggle(MNID_FULLSCREEN,_("Fullscreen"), g_config->use_fullscreen)
     ->set_help(_("Fill the entire screen"));
 
@@ -95,7 +94,7 @@ OptionsMenu::OptionsMenu() :
 	magn.clear();
 	break;
       }
-      
+
       ++count;
     }
     if (!magn.empty()) //magnification not in our list but accept anyway
@@ -104,65 +103,59 @@ OptionsMenu::OptionsMenu() :
       magnification->list.push_back(magn);
     }
   }
-  
 
-  SDL_Rect** modes = SDL_ListModes(NULL, SDL_FULLSCREEN|SDL_OPENGL);
-
-  if (modes == (SDL_Rect **)0) 
-  { // No resolutions at all available, bad
-
-  }
-  else if(modes == (SDL_Rect **)-1) 
-  { // All resolutions should work, so we fall back to hardcoded defaults
-    fullscreen_res->list.push_back("640x480");
-    fullscreen_res->list.push_back("800x600");
-    fullscreen_res->list.push_back("1024x768");
-    fullscreen_res->list.push_back("1152x864");
-    fullscreen_res->list.push_back("1280x960");
-    fullscreen_res->list.push_back("1280x1024");
-    fullscreen_res->list.push_back("1440x900");
-    fullscreen_res->list.push_back("1680x1050");
-    fullscreen_res->list.push_back("1600x1200");
-    fullscreen_res->list.push_back("1920x1080");
-    fullscreen_res->list.push_back("1920x1200");
-  }
-  else 
+  int display_mode_count = SDL_GetNumDisplayModes(0);
+  std::string last_display_mode;
+  for(int i = 0; i < display_mode_count; ++i)
   {
-    for(int i = 0; modes[i]; ++i)
+    SDL_DisplayMode mode;
+    int ret = SDL_GetDisplayMode(0, i, &mode);
+    if (ret != 0)
     {
-      std::ostringstream out;          
-      out << modes[i]->w << "x" << modes[i]->h;
+      log_warning << "failed to get display mode: " << SDL_GetError() << std::endl;
+    }
+    else
+    {
+      std::ostringstream out;
+      out << mode.w << "x" << mode.h << "@" << mode.refresh_rate;
+      if(last_display_mode == out.str())
+        continue;
+      last_display_mode = out.str();
       fullscreen_res->list.push_back(out.str());
     }
-
-    // On Ubuntu/Linux resolutions are returned from highest to
-    // lowest, so reverse them
-    std::sort(fullscreen_res->list.begin(), fullscreen_res->list.end(), StringUtil::numeric_less);
   }
-  
-  std::ostringstream out;
-  out << g_config->fullscreen_size.width << "x" << g_config->fullscreen_size.height;
-  std::string fllscrn_sz = out.str();
-  size_t cnt = 0;
-  for (std::vector<std::string>::iterator i = fullscreen_res->list.begin(); i != fullscreen_res->list.end(); ++i) 
+  fullscreen_res->list.push_back("Desktop");
+
+  std::string fullscreen_size_str = "Desktop";
   {
-    if (*i == fllscrn_sz)
+    std::ostringstream out;
+    if (g_config->fullscreen_size != Size(0, 0))
     {
-      fllscrn_sz.clear();
+      out << g_config->fullscreen_size.width << "x" << g_config->fullscreen_size.height << "@" << g_config->fullscreen_refresh_rate;
+      fullscreen_size_str = out.str();
+    }
+  }
+
+  size_t cnt = 0;
+  for (std::vector<std::string>::iterator i = fullscreen_res->list.begin(); i != fullscreen_res->list.end(); ++i)
+  {
+    if (*i == fullscreen_size_str)
+    {
+      fullscreen_size_str.clear();
       fullscreen_res->selected = cnt;
       break;
     }
     ++cnt;
   }
-  if (!fllscrn_sz.empty())
+  if (!fullscreen_size_str.empty())
   {
     fullscreen_res->selected = fullscreen_res->list.size();
-    fullscreen_res->list.push_back(fllscrn_sz);
+    fullscreen_res->list.push_back(fullscreen_size_str);
   }
 
   MenuItem* aspect = add_string_select(MNID_ASPECTRATIO, _("Aspect Ratio"));
   aspect->set_help(_("Adjust the aspect ratio"));
-  
+
   aspect->list.push_back(_("auto"));
   aspect->list.push_back("5:4");
   aspect->list.push_back("4:3");
@@ -175,16 +168,16 @@ OptionsMenu::OptionsMenu() :
     std::ostringstream out;
     out << g_config->aspect_size.width << ":" << g_config->aspect_size.height;
     std::string aspect_ratio = out.str();
-    size_t cnt = 0;
+    size_t cnt_ = 0;
     for(std::vector<std::string>::iterator i = aspect->list.begin(); i != aspect->list.end(); ++i)
     {
       if(*i == aspect_ratio)
       {
         aspect_ratio.clear();
-	aspect->selected = cnt;
+	aspect->selected = cnt_;
         break;
       }
-      ++cnt;
+      ++cnt_;
     }
 
     if (!aspect_ratio.empty())
@@ -193,8 +186,8 @@ OptionsMenu::OptionsMenu() :
       aspect->list.push_back(aspect_ratio);
     }
   }
-  
-  if (sound_manager->is_audio_enabled()) {
+
+  if (SoundManager::current()->is_audio_enabled()) {
     add_toggle(MNID_SOUND, _("Sound"), g_config->sound_enabled)
       ->set_help(_("Disable all sound effects"));
     add_toggle(MNID_MUSIC, _("Music"), g_config->music_enabled)
@@ -203,12 +196,23 @@ OptionsMenu::OptionsMenu() :
     add_inactive(MNID_SOUND, _("Sound (disabled)"));
     add_inactive(MNID_MUSIC, _("Music (disabled)"));
   }
-  
-  add_submenu(_("Setup Keyboard"), MenuStorage::get_key_options_menu())
+
+  add_submenu(_("Setup Keyboard"), MenuStorage::KEYBOARD_MENU)
     ->set_help(_("Configure key-action mappings"));
 
-  add_submenu(_("Setup Joystick"), MenuStorage::get_joystick_options_menu())
+  add_submenu(_("Setup Joystick"), MenuStorage::JOYSTICK_MENU)
     ->set_help(_("Configure joystick control-action mappings"));
+
+  if (g_config->developer_mode)
+  {
+    add_toggle(MNID_DEVELOPER_MODE, _("Developer Mode"), g_config->developer_mode);
+  }
+
+  if (g_config->is_christmas() || g_config->christmas_mode)
+  {
+    add_toggle(MNID_CHRISTMAS_MODE, _("Christmas Mode"), g_config->christmas_mode);
+  }
+
   add_hl();
   add_back(_("Back"));
 }
@@ -222,52 +226,66 @@ OptionsMenu::menu_action(MenuItem* item)
 {
   switch (item->id) {
     case MNID_ASPECTRATIO:
-    { 
-      if (item->list[item->selected] == _("auto"))
       {
-        g_config->aspect_size = Size(0, 0); // Magic values
-        Renderer::instance()->apply_config();
-        MenuManager::recalc_pos();
+        if (item->list[item->selected] == _("auto"))
+        {
+          g_config->aspect_size = Size(0, 0); // Magic values
+          VideoSystem::current()->get_renderer().apply_config();
+          MenuManager::instance().on_window_resize();
+        }
+        else if (sscanf(item->list[item->selected].c_str(), "%d:%d",
+                        &g_config->aspect_size.width, &g_config->aspect_size.height) == 2)
+        {
+          VideoSystem::current()->get_renderer().apply_config();
+          MenuManager::instance().on_window_resize();
+        }
+        else
+        {
+          assert(!"This must not be reached");
+        }
       }
-      else if (sscanf(item->list[item->selected].c_str(), "%d:%d", 
-                      &g_config->aspect_size.width, &g_config->aspect_size.height) == 2)
-      {
-        Renderer::instance()->apply_config();
-        MenuManager::recalc_pos();
-      }
-      else
-      {
-        assert(!"This must not be reached");
-      }
-    }
-    break;
+      break;
 
     case MNID_MAGNIFICATION:
       if (item->list[item->selected] == _("auto"))
       {
-        g_config->magnification = 0.0f; // Magic value 
+        g_config->magnification = 0.0f; // Magic value
       }
       else if(sscanf(item->list[item->selected].c_str(), "%f", &g_config->magnification) == 1)
       {
         g_config->magnification /= 100.0f;
       }
-      Renderer::instance()->apply_config();
-      MenuManager::recalc_pos();
+      VideoSystem::current()->get_renderer().apply_config();
+      MenuManager::instance().on_window_resize();
       break;
 
     case MNID_FULLSCREEN_RESOLUTION:
-      if(sscanf(item->list[item->selected].c_str(), "%dx%d", 
-                &g_config->fullscreen_size.width, &g_config->fullscreen_size.height) == 2)
       {
-        // do nothing, changes are only applied when toggling fullscreen mode
-      }      
+        int width;
+        int height;
+        int refresh_rate;
+        if (item->list[item->selected] == "Desktop")
+        {
+          g_config->fullscreen_size.width = 0;
+          g_config->fullscreen_size.height = 0;
+          g_config->fullscreen_refresh_rate = 0;
+        }
+        else if(sscanf(item->list[item->selected].c_str(), "%dx%d@%d",
+                  &width, &height, &refresh_rate) == 3)
+        {
+          // do nothing, changes are only applied when toggling fullscreen mode
+          g_config->fullscreen_size.width = width;
+          g_config->fullscreen_size.height = height;
+          g_config->fullscreen_refresh_rate = refresh_rate;
+        }
+      }
       break;
 
     case MNID_FULLSCREEN:
       if(g_config->use_fullscreen != is_toggled(MNID_FULLSCREEN)) {
         g_config->use_fullscreen = !g_config->use_fullscreen;
-        Renderer::instance()->apply_config();
-        MenuManager::recalc_pos();
+        VideoSystem::current()->get_renderer().apply_config();
+        MenuManager::instance().on_window_resize();
         g_config->save();
       }
       break;
@@ -275,7 +293,7 @@ OptionsMenu::menu_action(MenuItem* item)
     case MNID_SOUND:
       if(g_config->sound_enabled != is_toggled(MNID_SOUND)) {
         g_config->sound_enabled = !g_config->sound_enabled;
-        sound_manager->enable_sound(g_config->sound_enabled);
+        SoundManager::current()->enable_sound(g_config->sound_enabled);
         g_config->save();
       }
       break;
@@ -283,19 +301,24 @@ OptionsMenu::menu_action(MenuItem* item)
     case MNID_MUSIC:
       if(g_config->music_enabled != is_toggled(MNID_MUSIC)) {
         g_config->music_enabled = !g_config->music_enabled;
-        sound_manager->enable_music(g_config->music_enabled);
+        SoundManager::current()->enable_music(g_config->music_enabled);
         g_config->save();
       }
       break;
 
+    case MNID_DEVELOPER_MODE:
+      g_config->developer_mode = is_toggled(MNID_DEVELOPER_MODE);
+      log_info << "developer mode: " << g_config->developer_mode << std::endl;
+      break;
+
+    case MNID_CHRISTMAS_MODE:
+      g_config->christmas_mode = is_toggled(MNID_CHRISTMAS_MODE);
+      log_info << "christmas mode: " << g_config->christmas_mode << std::endl;
+      break;
+
     default:
       break;
   }
 }
 
-void
-OptionsMenu::check_menu()
-{
-}
-
 /* EOF */
