@@ -18,22 +18,15 @@
 #ifndef HEADER_SUPERTUX_SUPERTUX_OPTIONS_MENU_HPP
 #define HEADER_SUPERTUX_SUPERTUX_OPTIONS_MENU_HPP
 
-
 #include "gui/menu.hpp"
 
-class LanguageMenu;
-
 class OptionsMenu : public Menu
 {
 public:
-  OptionsMenu();
+  OptionsMenu(bool complete);
   virtual ~OptionsMenu();
 
-  void menu_action(MenuItem* item);
-  void check_menu();
-
-protected:
-  std::auto_ptr<LanguageMenu> language_menu; 
+  void menu_action(MenuItem* item) override;
 };
 
 #endif
