@@ -1,5 +1,5 @@
 //  SuperTux -  A Jump'n Run
-//  Copyright (C) 2004 Ingo Ruhnke <grumbel@gmx.de>
+//  Copyright (C) 2004 Ingo Ruhnke <grumbel@gmail.com>
 //  Copyright (C) 2006 Christoph Sommer <christoph.sommer@2006.expires.deltadevelopment.de>
 //
 //  This program is free software: you can redistribute it and/or modify
@@ -25,11 +25,10 @@
 #include <physfs.h>
 #include <sstream>
 #include <stdexcept>
-#include <unistd.h>
 #include <vector>
 
 #include "audio/sound_manager.hpp"
-#include "control/joystickkeyboardcontroller.hpp"
+#include "control/input_manager.hpp"
 #include "gui/menu.hpp"
 #include "gui/menu_manager.hpp"
 #include "gui/mousecursor.hpp"
@@ -39,19 +38,22 @@
 #include "object/background.hpp"
 #include "object/decal.hpp"
 #include "object/tilemap.hpp"
-#include "physfs/ifile_stream.hpp"
+#include "physfs/ifile_streambuf.hpp"
+#include "scripting/scripting.hpp"
 #include "scripting/squirrel_error.hpp"
 #include "scripting/squirrel_util.hpp"
 #include "sprite/sprite.hpp"
 #include "sprite/sprite_manager.hpp"
 #include "supertux/game_session.hpp"
+#include "supertux/gameconfig.hpp"
 #include "supertux/globals.hpp"
-#include "supertux/screen_manager.hpp"
 #include "supertux/menu/menu_storage.hpp"
 #include "supertux/menu/options_menu.hpp"
 #include "supertux/menu/worldmap_menu.hpp"
 #include "supertux/player_status.hpp"
 #include "supertux/resources.hpp"
+#include "supertux/savegame.hpp"
+#include "supertux/screen_manager.hpp"
 #include "supertux/sector.hpp"
 #include "supertux/shrinkfade.hpp"
 #include "supertux/spawn_point.hpp"
@@ -77,12 +79,11 @@ namespace worldmap {
 
 WorldMap* WorldMap::current_ = NULL;
 
-WorldMap::WorldMap(const std::string& filename, PlayerStatus* player_status, const std::string& force_spawnpoint) :
-  tux(0),
-  player_status(player_status),
-  tileset(NULL), 
+WorldMap::WorldMap(const std::string& filename, Savegame& savegame, const std::string& force_spawnpoint_) :
+  tux(),
+  m_savegame(savegame),
+  tileset(NULL),
   free_tileset(false),
-  worldmap_menu(),
   camera_offset(),
   name(),
   music(),
@@ -101,13 +102,14 @@ WorldMap::WorldMap(const std::string& filename, PlayerStatus* player_status, con
   total_stats(),
   worldmap_table(),
   scripts(),
-  ambient_light( 1.0f, 1.0f, 1.0f, 1.0f ), 
-  force_spawnpoint(force_spawnpoint),
-  in_level(false), 
+  ambient_light( 1.0f, 1.0f, 1.0f, 1.0f ),
+  force_spawnpoint(force_spawnpoint_),
+  in_level(false),
   pan_pos(),
-  panning(false)
+  panning(false),
+  last_position()
 {
-  tux = new Tux(this);
+  tux = std::make_shared<Tux>(this);
   add_object(tux);
 
   name = "<no title>";
@@ -115,8 +117,6 @@ WorldMap::WorldMap(const std::string& filename, PlayerStatus* player_status, con
 
   total_stats.reset();
 
-  worldmap_menu.reset(new WorldmapMenu());
-
   // create a new squirrel table for the worldmap
   using namespace scripting;
 
@@ -133,8 +133,8 @@ WorldMap::WorldMap(const std::string& filename, PlayerStatus* player_status, con
   sq_addref(global_vm, &worldmap_table);
   sq_pop(global_vm, 1);
 
-  sound_manager->preload("sounds/warp.wav");
-  
+  SoundManager::current()->preload("sounds/warp.wav");
+
   // load worldmap objects
   load(filename);
 }
@@ -148,9 +148,8 @@ WorldMap::~WorldMap()
 
   for(GameObjects::iterator i = game_objects.begin();
       i != game_objects.end(); ++i) {
-    GameObject* object = *i;
+    GameObjectPtr& object = *i;
     try_unexpose(object);
-    object->unref();
   }
 
   for(SpawnPoints::iterator i = spawn_points.begin();
@@ -172,22 +171,21 @@ WorldMap::~WorldMap()
 }
 
 void
-WorldMap::add_object(GameObject* object)
+WorldMap::add_object(GameObjectPtr object)
 {
-  TileMap* tilemap = dynamic_cast<TileMap*> (object);
+  TileMap* tilemap = dynamic_cast<TileMap*>(object.get());
   if(tilemap != 0 && tilemap->is_solid()) {
     solid_tilemaps.push_back(tilemap);
   }
 
-  object->ref();
   try_expose(object);
   game_objects.push_back(object);
 }
 
 void
-WorldMap::try_expose(GameObject* object)
+WorldMap::try_expose(const GameObjectPtr& object)
 {
-  ScriptInterface* object_ = dynamic_cast<ScriptInterface*> (object);
+  ScriptInterface* object_ = dynamic_cast<ScriptInterface*>(object.get());
   if(object_ != NULL) {
     HSQUIRRELVM vm = scripting::global_vm;
     sq_pushobject(vm, worldmap_table);
@@ -197,9 +195,9 @@ WorldMap::try_expose(GameObject* object)
 }
 
 void
-WorldMap::try_unexpose(GameObject* object)
+WorldMap::try_unexpose(const GameObjectPtr& object)
 {
-  ScriptInterface* object_ = dynamic_cast<ScriptInterface*> (object);
+  ScriptInterface* object_ = dynamic_cast<ScriptInterface*>(object.get());
   if(object_ != NULL) {
     HSQUIRRELVM vm = scripting::global_vm;
     SQInteger oldtop = sq_gettop(vm);
@@ -237,10 +235,10 @@ WorldMap::move_to_spawnpoint(const std::string& spawnpoint, bool pan)
 }
 
 void
-WorldMap::change(const std::string& filename, const std::string& force_spawnpoint)
+WorldMap::change(const std::string& filename, const std::string& force_spawnpoint_)
 {
-  g_screen_manager->exit_screen();
-  g_screen_manager->push_screen(new WorldMap(filename, player_status, force_spawnpoint));
+  ScreenManager::current()->pop_screen();
+  ScreenManager::current()->push_screen(std::unique_ptr<Screen>(new WorldMap(filename, m_savegame, force_spawnpoint_)));
 }
 
 void
@@ -253,41 +251,41 @@ WorldMap::load(const std::string& filename)
     lisp::Parser parser;
     const lisp::Lisp* root = parser.parse(map_filename);
 
-    const lisp::Lisp* level = root->get_lisp("supertux-level");
-    if(level == NULL)
+    const lisp::Lisp* level_ = root->get_lisp("supertux-level");
+    if(level_ == NULL)
       throw std::runtime_error("file isn't a supertux-level file.");
 
-    level->get("name", name);
+    level_->get("name", name);
 
-    const lisp::Lisp* sector = level->get_lisp("sector");
+    const lisp::Lisp* sector = level_->get_lisp("sector");
     if(!sector)
       throw std::runtime_error("No sector specified in worldmap file.");
 
-    const lisp::Lisp* tilesets_lisp = level->get_lisp("tilesets");
+    const lisp::Lisp* tilesets_lisp = level_->get_lisp("tilesets");
     if(tilesets_lisp != NULL) {
-      tileset      = tile_manager->parse_tileset_definition(*tilesets_lisp);
+      tileset      = TileManager::current()->parse_tileset_definition(*tilesets_lisp).release();
       free_tileset = true;
     }
     std::string tileset_name;
-    if(level->get("tileset", tileset_name)) {
+    if(level_->get("tileset", tileset_name)) {
       if(tileset != NULL) {
-        log_warning << "multiple tilesets specified in level" << std::endl;
+        log_warning << "multiple tilesets specified in level_" << std::endl;
       } else {
-        tileset = tile_manager->get_tileset(tileset_name);
+        tileset = TileManager::current()->get_tileset(tileset_name);
       }
     }
     /* load default tileset */
     if(tileset == NULL) {
-      tileset = tile_manager->get_tileset("images/worldmap.strf");
+      tileset = TileManager::current()->get_tileset("images/worldmap.strf");
     }
     current_tileset = tileset;
 
     lisp::ListIterator iter(sector);
     while(iter.next()) {
       if(iter.item() == "tilemap") {
-        add_object(new TileMap(*(iter.lisp())));
+        add_object(std::make_shared<TileMap>(*(iter.lisp())));
       } else if(iter.item() == "background") {
-        add_object(new Background(*(iter.lisp())));
+        add_object(std::make_shared<Background>(*(iter.lisp())));
       } else if(iter.item() == "music") {
         iter.value()->get(music);
       } else if(iter.item() == "init-script") {
@@ -296,23 +294,23 @@ WorldMap::load(const std::string& filename)
         SpawnPoint* sp = new SpawnPoint(*iter.lisp());
         spawn_points.push_back(sp);
       } else if(iter.item() == "level") {
-        LevelTile* level = new LevelTile(levels_path, *iter.lisp());
-        levels.push_back(level);
+        auto level = std::make_shared<LevelTile>(levels_path, *iter.lisp());
+        levels.push_back(level.get());
         add_object(level);
       } else if(iter.item() == "special-tile") {
-        SpecialTile* special_tile = new SpecialTile(*iter.lisp());
-        special_tiles.push_back(special_tile);
+        auto special_tile = std::make_shared<SpecialTile>(*iter.lisp());
+        special_tiles.push_back(special_tile.get());
         add_object(special_tile);
       } else if(iter.item() == "sprite-change") {
-        SpriteChange* sprite_change = new SpriteChange(*iter.lisp());
-        sprite_changes.push_back(sprite_change);
+        auto sprite_change = std::make_shared<SpriteChange>(*iter.lisp());
+        sprite_changes.push_back(sprite_change.get());
         add_object(sprite_change);
       } else if(iter.item() == "teleporter") {
-        Teleporter* teleporter = new Teleporter(*iter.lisp());
-        teleporters.push_back(teleporter);
+        auto teleporter = std::make_shared<Teleporter>(*iter.lisp());
+        teleporters.push_back(teleporter.get());
         add_object(teleporter);
       } else if(iter.item() == "decal") {
-        Decal* decal = new Decal(*iter.lisp());
+        auto decal = std::make_shared<Decal>(*iter.lisp());
         add_object(decal);
       } else if(iter.item() == "ambient-light") {
         std::vector<float> vColor;
@@ -330,7 +328,7 @@ WorldMap::load(const std::string& filename)
     }
     current_tileset = NULL;
 
-    if(solid_tilemaps.size() == 0)
+    if(solid_tilemaps.empty())
       throw std::runtime_error("No solid tilemap specified");
 
     move_to_spawnpoint("main");
@@ -344,10 +342,17 @@ WorldMap::load(const std::string& filename)
 }
 
 void
-WorldMap::get_level_title(LevelTile& level)
+WorldMap::load_level_information(LevelTile& level)
 {
+  if(last_position == tux->get_tile_pos())
+  {
+    return;
+  }
+
+  last_position = tux->get_tile_pos();
   /** get special_tile's title */
   level.title = "<no title>";
+  level.target_time = 0.0f;
 
   try {
     lisp::Parser parser;
@@ -358,8 +363,10 @@ WorldMap::get_level_title(LevelTile& level)
       return;
 
     level_lisp->get("name", level.title);
+    level_lisp->get("target-time", level.target_time);
+
   } catch(std::exception& e) {
-    log_warning << "Problem when reading leveltitle: " << e.what() << std::endl;
+    log_warning << "Problem when reading level information: " << e.what() << std::endl;
     return;
   }
 }
@@ -379,11 +386,11 @@ void
 WorldMap::on_escape_press()
 {
   // Show or hide the menu
-  if(!MenuManager::current()) {
-    MenuManager::set_current(worldmap_menu.get());
+  if(!MenuManager::instance().is_active()) {
+    MenuManager::instance().set_menu(MenuStorage::WORLDMAP_MENU);
     tux->set_direction(D_NONE);  // stop tux movement when menu is called
   } else {
-    MenuManager::set_current(NULL);
+    MenuManager::instance().clear_menu_stack();
   }
 }
 
@@ -454,6 +461,10 @@ WorldMap::finished_level(Level* gamelevel)
   // TODO use Level* parameter here?
   LevelTile* level = at_level();
 
+  if(level == NULL) {
+    return;
+  }
+
   bool old_level_state = level->solved;
   level->solved = true;
   level->sprite->set_action("solved");
@@ -461,6 +472,14 @@ WorldMap::finished_level(Level* gamelevel)
   // deal with statistics
   level->statistics.merge(gamelevel->stats);
   calculate_total_stats();
+  if (level->target_time == 0.0f)
+    load_level_information(*level);
+
+  if(level->statistics.completed(level->statistics, level->target_time)) {
+    level->perfect = true;
+    if(level->sprite->has_action("perfect"))
+      level->sprite->set_action("perfect");
+  }
 
   save_state();
 
@@ -507,11 +526,11 @@ WorldMap::finished_level(Level* gamelevel)
 
 Vector
 WorldMap::get_camera_pos_for_tux() {
-  Vector camera_offset;
+  Vector camera_offset_;
   Vector tux_pos = tux->get_pos();
-  camera_offset.x = tux_pos.x - SCREEN_WIDTH/2;
-  camera_offset.y = tux_pos.y - SCREEN_HEIGHT/2;
-  return camera_offset;
+  camera_offset_.x = tux_pos.x - SCREEN_WIDTH/2;
+  camera_offset_.y = tux_pos.y - SCREEN_HEIGHT/2;
+  return camera_offset_;
 }
 
 void
@@ -535,27 +554,11 @@ WorldMap::clamp_camera_position(Vector& c) {
 void
 WorldMap::update(float delta)
 {
-  if(!in_level) {
-    Menu* menu = MenuManager::current();
-    if(menu != NULL) {
-      if(menu == worldmap_menu.get()) {
-        switch (worldmap_menu->check())
-        {
-          case MNID_RETURNWORLDMAP: // Return to game
-            MenuManager::set_current(0);
-            break;
-          case MNID_QUITWORLDMAP: // Quit Worldmap
-            g_screen_manager->exit_screen();
-            break;
-        }
-      }
-
-      return;
-    }
-
+  if (!in_level && !MenuManager::instance().is_active())
+  {
     // update GameObjects
     for(size_t i = 0; i < game_objects.size(); ++i) {
-      GameObject* object = game_objects[i];
+      GameObjectPtr& object = game_objects[i];
       if(!panning || object != tux) {
         object->update(delta);
       }
@@ -564,10 +567,9 @@ WorldMap::update(float delta)
     // remove old GameObjects
     for(GameObjects::iterator i = game_objects.begin();
         i != game_objects.end(); ) {
-      GameObject* object = *i;
+      GameObjectPtr& object = *i;
       if(!object->is_valid()) {
         try_unexpose(object);
-        object->unref();
         i = game_objects.erase(i);
       } else {
         ++i;
@@ -577,10 +579,9 @@ WorldMap::update(float delta)
     /* update solid_tilemaps list */
     //FIXME: this could be more efficient
     solid_tilemaps.clear();
-    for(std::vector<GameObject*>::iterator i = game_objects.begin();
-        i != game_objects.end(); ++i)
+    for(auto i = game_objects.begin(); i != game_objects.end(); ++i)
     {
-      TileMap* tm = dynamic_cast<TileMap*>(*i);
+      TileMap* tm = dynamic_cast<TileMap*>(i->get());
       if (!tm) continue;
       if (tm->is_solid()) solid_tilemaps.push_back(tm);
     }
@@ -591,12 +592,12 @@ WorldMap::update(float delta)
     if(!panning) {
       camera_offset = get_camera_pos_for_tux();
     } else {
-      Vector delta = pan_pos - camera_offset;
-      float mag = delta.norm();
+      Vector delta__ = pan_pos - camera_offset;
+      float mag = delta__.norm();
       if(mag > CAMERA_PAN_SPEED) {
-        delta *= CAMERA_PAN_SPEED/mag;
+        delta__ *= CAMERA_PAN_SPEED/mag;
       }
-      camera_offset += delta;
+      camera_offset += delta__;
       if(camera_offset == pan_pos) {
         panning = false;
       }
@@ -615,7 +616,7 @@ WorldMap::update(float delta)
     }
 
     // handle input
-    Controller *controller = g_jk_controller->get_main_controller();
+    Controller *controller = InputManager::current()->get_controller();
     bool enter_level = false;
     if(controller->pressed(Controller::ACTION)
        || controller->pressed(Controller::JUMP)
@@ -624,8 +625,17 @@ WorldMap::update(float delta)
       if(!controller->pressed(Controller::UP))
         enter_level = true;
     }
-    if(controller->pressed(Controller::PAUSE_MENU))
+    if(controller->pressed(Controller::START) ||
+       controller->pressed(Controller::ESCAPE))
+    {
       on_escape_press();
+    }
+
+    if(controller->pressed(Controller::CHEAT_MENU) &&
+       g_config->developer_mode)
+    {
+      MenuManager::instance().set_menu(MenuStorage::WORLDMAP_CHEAT_MENU);
+    }
 
     // check for teleporters
     Teleporter* teleporter = at_teleporter(tux->get_tile_pos());
@@ -635,7 +645,7 @@ WorldMap::update(float delta)
         change(teleporter->worldmap, teleporter->spawnpoint);
       } else {
         // TODO: an animation, camera scrolling or a fading would be a nice touch
-        sound_manager->play("sounds/warp.wav");
+        SoundManager::current()->play("sounds/warp.wav");
         tux->back_direction = D_NONE;
         move_to_spawnpoint(teleporter->spawnpoint, true);
       }
@@ -645,13 +655,15 @@ WorldMap::update(float delta)
     LevelTile* level = at_level();
     if (level && (level->auto_play) && (!level->solved) && (!tux->is_moving())) {
       enter_level = true;
+      // automatically mark these levels as solved in case player aborts
+      level->solved = true;
     }
 
     if (enter_level && !tux->is_moving())
     {
       /* Check level action */
-      LevelTile* level = at_level();
-      if (!level) {
+      LevelTile* level_ = at_level();
+      if (!level_) {
         //Respawn if player on a tile with no level and nowhere to go.
         int tile_data = tile_data_at(tux->get_tile_pos());
         if(!( tile_data & ( Tile::WORLDMAP_NORTH |  Tile::WORLDMAP_SOUTH | Tile::WORLDMAP_WEST | Tile::WORLDMAP_EAST ))){
@@ -663,17 +675,17 @@ WorldMap::update(float delta)
         return;
       }
 
-      if (level->pos == tux->get_tile_pos()) {
+      if (level_->pos == tux->get_tile_pos()) {
         try {
-          Vector shrinkpos = Vector(level->pos.x*32 + 16 - camera_offset.x,
-                                    level->pos.y*32 +  8 - camera_offset.y);
-          std::string levelfile = levels_path + level->get_name();
+          Vector shrinkpos = Vector(level_->pos.x*32 + 16 - camera_offset.x,
+                                    level_->pos.y*32 +  8 - camera_offset.y);
+          std::string levelfile = levels_path + level_->get_name();
 
           // update state and savegame
           save_state();
 
-          g_screen_manager->push_screen(new GameSession(levelfile, player_status, &level->statistics),
-                                   new ShrinkFade(shrinkpos, 1.0f));
+          ScreenManager::current()->push_screen(std::unique_ptr<Screen>(new GameSession(levelfile, m_savegame, &level_->statistics)),
+                                        std::unique_ptr<ScreenFade>(new ShrinkFade(shrinkpos, 1.0f)));
           in_level = true;
         } catch(std::exception& e) {
           log_fatal << "Couldn't load level: " << e.what() << std::endl;
@@ -692,7 +704,7 @@ WorldMap::tile_data_at(Vector p)
 {
   int dirs = 0;
 
-  for(std::list<TileMap*>::const_iterator i = solid_tilemaps.begin(); i != solid_tilemaps.end(); i++) {
+  for(std::list<TileMap*>::const_iterator i = solid_tilemaps.begin(); i != solid_tilemaps.end(); ++i) {
     TileMap* tilemap = *i;
     const Tile* tile = tilemap->get_tile((int)p.x, (int)p.y);
     int dirdata = tile->getData();
@@ -768,9 +780,9 @@ WorldMap::draw(DrawingContext& context)
   context.push_transform();
   context.set_translation(camera_offset);
 
-  for(GameObjects::iterator i = game_objects.begin();
-      i != game_objects.end(); ++i) {
-    GameObject* object = *i;
+  for(auto i = game_objects.begin(); i != game_objects.end(); ++i)
+  {
+    GameObjectPtr& object = *i;
     if(!panning || object != tux) {
       object->draw(context);
     }
@@ -779,7 +791,7 @@ WorldMap::draw(DrawingContext& context)
   /*
   // FIXME: make this a runtime switch similar to draw_collrects/show_collrects?
   // draw visual indication of possible walk directions
-  static int flipme = 0; 
+  static int flipme = 0;
   if (flipme++ & 0x04)
   for (int x = 0; x < get_width(); x++) {
   for (int y = 0; y < get_height(); y++) {
@@ -807,7 +819,7 @@ WorldMap::draw_status(DrawingContext& context)
   context.push_transform();
   context.set_translation(Vector(0, 0));
 
-  player_status->draw(context);
+  m_savegame.get_player_status()->draw(context);
 
   if (!tux->is_moving()) {
     for(LevelTiles::iterator i = levels.begin(); i != levels.end(); ++i) {
@@ -815,12 +827,12 @@ WorldMap::draw_status(DrawingContext& context)
 
       if (level->pos == tux->get_tile_pos()) {
         if(level->title == "")
-          get_level_title(*level);
+          load_level_information(*level);
 
         context.draw_text(Resources::normal_font, level->title,
                           Vector(SCREEN_WIDTH/2,
-                                 SCREEN_HEIGHT - Resources::normal_font->get_height() - 30),
-                          ALIGN_CENTER, LAYER_FOREGROUND1, WorldMap::level_title_color);
+                                 SCREEN_HEIGHT - Resources::normal_font->get_height() - 10),
+                          ALIGN_CENTER, LAYER_HUD, WorldMap::level_title_color);
 
         // if level is solved, draw level picture behind stats
         /*
@@ -835,7 +847,9 @@ WorldMap::draw_status(DrawingContext& context)
           }
         */
 
-        level->statistics.draw_worldmap_info(context);
+        if (level->target_time == 0.0f)
+          load_level_information(*level);
+        level->statistics.draw_worldmap_info(context, level->target_time);
         break;
       }
     }
@@ -876,8 +890,8 @@ WorldMap::draw_status(DrawingContext& context)
 void
 WorldMap::setup()
 {
-  sound_manager->play_music(music);
-  MenuManager::set_current(NULL);
+  SoundManager::current()->play_music(music);
+  MenuManager::instance().clear_menu_stack();
 
   current_ = this;
   load_state();
@@ -902,7 +916,8 @@ WorldMap::setup()
 
   //Run default.nut just before init script
   try {
-    IFileStream in(levels_path + "/default.nut");
+    IFileStreambuf ins(levels_path + "default.nut");
+    std::istream in(&ins);
     run_script(in, "WorldMap::default.nut");
   } catch(std::exception& ) {
     // doesn't exist or erroneous; do nothing
@@ -930,6 +945,16 @@ WorldMap::leave()
   sq_pop(global_vm, 1);
 }
 
+void
+WorldMap::set_levels_solved(bool solved, bool perfect)
+{
+  for(auto& level : levels)
+  {
+    level->set_solved(solved);
+    level->set_perfect(perfect);
+  }
+}
+
 void
 WorldMap::save_state()
 {
@@ -960,7 +985,8 @@ WorldMap::save_state()
 
     sq_pushstring(vm, map_filename.c_str(), map_filename.length());
     if(SQ_FAILED(sq_deleteslot(vm, -2, SQFalse)))
-      sq_pop(vm, 1);
+    {
+    }
 
     // construct new table for this worldmap
     sq_pushstring(vm, map_filename.c_str(), map_filename.length());
@@ -987,26 +1013,35 @@ WorldMap::save_state()
       sq_newtable(vm);
 
       store_bool(vm, "solved", level->solved);
+      store_bool(vm, "perfect", level->perfect);
       level->statistics.serialize_to_squirrel(vm);
 
-      sq_createslot(vm, -3);
+      if(SQ_FAILED(sq_createslot(vm, -3)))
+      {
+        throw std::runtime_error("failed to create '" + name + "' table entry");
+      }
     }
 
-    sq_createslot(vm, -3);
+    if(SQ_FAILED(sq_createslot(vm, -3)))
+    {
+      throw std::runtime_error("failed to create '" + name + "' table entry");
+    }
 
     // overall statistics...
     total_stats.serialize_to_squirrel(vm);
 
     // push world into worlds table
-    sq_createslot(vm, -3);
+    if(SQ_FAILED(sq_createslot(vm, -3)))
+    {
+      throw std::runtime_error("failed to create '" + name + "' table entry");
+    }
   } catch(std::exception& ) {
     sq_settop(vm, oldtop);
   }
 
   sq_settop(vm, oldtop);
 
-  if(World::current() != NULL)
-    World::current()->save_state();
+  m_savegame.save();
 }
 
 void
@@ -1058,7 +1093,11 @@ WorldMap::load_state()
       sq_pushstring(vm, level->get_name().c_str(), -1);
       if(SQ_SUCCEEDED(sq_get(vm, -2))) {
         level->solved = read_bool(vm, "solved");
-        level->sprite->set_action(level->solved ? "solved" : "default");
+        level->perfect = read_bool(vm, "perfect");
+        if(!level->solved)
+          level->sprite->set_action("default");
+        else
+          level->sprite->set_action((level->sprite->has_action("perfect") && level->perfect) ? "perfect" : "solved");
         level->statistics.unserialize_from_squirrel(vm);
         sq_pop(vm, 1);
       }
@@ -1079,7 +1118,7 @@ WorldMap::load_state()
 }
 
 size_t
-WorldMap::level_count()
+WorldMap::level_count() const
 {
   return levels.size();
 }
@@ -1136,7 +1175,7 @@ float
 WorldMap::get_width() const
 {
   float width = 0;
-  for(std::list<TileMap*>::const_iterator i = solid_tilemaps.begin(); i != solid_tilemaps.end(); i++) {
+  for(std::list<TileMap*>::const_iterator i = solid_tilemaps.begin(); i != solid_tilemaps.end(); ++i) {
     TileMap* solids = *i;
     if (solids->get_width() > width) width = solids->get_width();
   }
@@ -1147,7 +1186,7 @@ float
 WorldMap::get_height() const
 {
   float height = 0;
-  for(std::list<TileMap*>::const_iterator i = solid_tilemaps.begin(); i != solid_tilemaps.end(); i++) {
+  for(std::list<TileMap*>::const_iterator i = solid_tilemaps.begin(); i != solid_tilemaps.end(); ++i) {
     TileMap* solids = *i;
     if (solids->get_height() > height) height = solids->get_height();
   }
